INTERFACE
SUBROUTINE SEAPRE(PARA,KPARA,PSTPRE,KLEV)
USE PARKIND1 ,ONLY : JPIM ,JPRB
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
REAL(KIND=JPRB) ,INTENT(IN) :: PARA
INTEGER(KIND=JPIM),INTENT(OUT) :: KPARA
REAL(KIND=JPRB) ,INTENT(IN) :: PSTPRE(KLEV)
END SUBROUTINE SEAPRE
END INTERFACE
