program test_congvec
  USE COSP_KINDS, ONLY: wp
  ! *****************************COPYRIGHT****************************
  ! (c) British Crown Copyright 2009, the Met Office.
  ! All rights reserved.
  ! $Revision: 23 $, $Date: 2011-03-31 07:41:37 -0600 (Thu, 31 Mar 2011) $
  ! $URL: https://cfmip-obs-sim.googlecode.com/svn/devel/branches/dustinswales/icarus-scops-4.1-bsd/test_congvec.f $
  ! 
  ! Redistribution and use in source and binary forms, with or without 
  ! modification, are permitted provided that the
  ! following conditions are met:
  ! 
  !     * Redistributions of source code must retain the above 
  !       copyright  notice, this list of conditions and the following 
  !       disclaimer.
  !     * Redistributions in binary form must reproduce the above 
  !       copyright notice, this list of conditions and the following 
  !       disclaimer in the documentation and/or other materials 
  !       provided with the distribution.
  !     * Neither the name of the Met Office nor the names of its 
  !       contributors may be used to endorse or promote products
  !       derived from this software without specific prior written 
  !       permission.
  ! 
  ! THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  ! "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
  ! LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
  ! A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
  ! OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
  ! SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
  ! LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
  ! DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
  ! THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
  ! (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
  ! OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  
  ! 
  ! *****************************COPYRIGHT*******************************
  ! *****************************COPYRIGHT*******************************
  
  ! $Id: test_congvec.f,v 4.0 2009/02/13 08:12:22 hadmw Exp $
  
  implicit none
  integer, parameter :: huge32      = 2147483647, &
       npoints     = 100,        &
       niterations = 100,        &
       i2_16       = 65536
  
  integer j,i,irand,overflow_32
  integer,  dimension(npoints) :: seed
  real(wp), dimension(npoints) :: ran
  
  do j=1,npoints 
     seed(j)=j*129
  enddo
  
  do i=1,100 
     include 'congvec.f90'
     write (6,'(f8.3)') (ran(j),j=1,npoints)
  enddo

end program test_congvec
