MODULE raddimlw_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: nua, ntra, ninter, ng1, ng1p1

  INTEGER NUA
  PARAMETER (NUA = 24)
  INTEGER NTRA
  PARAMETER (NTRA = 15)
  INTEGER Ninter
  PARAMETER (Ninter = 6)
  INTEGER NG1, NG1P1
  PARAMETER (NG1 = 2, NG1P1 = NG1 + 1)
END MODULE raddimlw_mod_h
