MODULE YOMHOOK

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

! Used by "hook" function
! LHOOK = true implies "hook" function will be called
! Altough initialized to TRUE it will be reset by first call to 
! DR_HOOK unless we really want to use the hook function

LOGICAL :: LHOOK=.TRUE.

CONTAINS 

SUBROUTINE DR_HOOK(CDNAME,KSWITCH,PKEY)
CHARACTER(LEN=*), INTENT(IN) :: CDNAME
INTEGER(KIND=JPIM),        INTENT(IN) :: KSWITCH
REAL(KIND=JPRB),        INTENT(INOUT) :: PKEY
IF (KSWITCH.EQ.0) THEN
!  PRINT *,'--> ON ENTRE DANS: ',CDNAME
ELSE IF (KSWITCH.EQ.1) THEN
!  PRINT *,'    ON SORT  DE  : ',CDNAME
ENDIF
END SUBROUTINE DR_HOOK

SUBROUTINE C_DRHOOK_PRINT

! subroutine vide MPL 4.12.08
!INTEGER(KIND=JPIM) ITID, KTID, IPRINT_OPTION, ILEVEL
!ITID = 0
!KTID = 0
!IPRINT_OPTION = 2
!ILEVEL = 0
PRINT *,'ON EST DANS C_DRHOOK_PRINT '

END SUBROUTINE C_DRHOOK_PRINT
END MODULE YOMHOOK
