MODULE TPM_FFT
USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

TYPE FFT_TYPE
  REAL(KIND=JPRB)   ,ALLOCATABLE :: TRIGS(:,:)
  INTEGER(KIND=JPIM),ALLOCATABLE :: NFAX(:,:)
END TYPE FFT_TYPE

TYPE(FFT_TYPE),ALLOCATABLE,TARGET :: FFT_RESOL(:)
TYPE(FFT_TYPE),POINTER     :: T

!$OMP THREADPRIVATE(t)

END MODULE TPM_FFT
