      MODULE indice_sol_mod

            IMPLICIT NONE

            INTEGER, PARAMETER :: nbsrf = 4 ! nombre de sous-fractions pour une maille
            INTEGER, PARAMETER :: is_ter=1, is_lic=2, is_oce = 3, is_sic=4
                              ! terre ! ocean ! glacier continental ! glace de mer

            INTEGER, PARAMETER :: is_ave=nbsrf+1 ! valeur moyenne sur l'ensemble des surfaces
            REAL, PARAMETER :: epsfra=1.0E-05

            CHARACTER(len=3), DIMENSION(nbsrf), PARAMETER :: clnsurf = (/'ter', 'lic', 'oce', 'sic'/)
!FC 
           INTEGER, SAVE    :: nvm_orch ! Nombre de type de vegetation ds ORCHIDEE                 
           !$OMP THREADPRIVATE(nvm_orch)
!
!AM heterogeneous continental sub-surfaces
            !!! If max_nbtersrf is modified, please change also the output number in phys_output_ctrlout_mod.F90
            INTEGER, PARAMETER :: max_nbtersrf = 5  ! maximal number of continental sub-surfaces
            CHARACTER(len=1), DIMENSION(max_nbtersrf), PARAMETER :: nb_tersrf = (/'1', '2', '3', '4', '5'/)
            !!! nsoilout must be lower than nsoilmx
            INTEGER, PARAMETER :: nsoilout = 10  ! number of soil layers for output
            CHARACTER(len=2), DIMENSION(nsoilout), PARAMETER :: nb_soil = (/'01','02','03','04','05','06','07','08','09','10'/)!,'11','12','13','14'/)

      END MODULE indice_sol_mod

