MODULE YOMVDOZ

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

! --------------------------------------------------------------------
!            L'OBJET DE CE  EST DE FOURNIR
!   LES VITESSES DE DEPOT DE L'OZONE.

! ELLES SERONT UTILISEES DANS LA SUBROUTINE ACDIFOZ
! CES VITESSES DE DEPOT DEPENDENT DES CLASSES DE VEGETATION,
! DE L'OCCURENCE NUIT-JOUR, DE L'OCCURENCE SEC-HUMIDE
! ET DE LA SAISON.

!   ===== SIGNIFICATION DES NOMS DES VARIABLES ======

! VDHJS : VITESSE DE DEPOT DE L'OZONE EN HIVER POUR UN JOUR SEC
! VDHJH : VITESSE DE DEPOT DE L'OZONE EN HIVER POUR UN JOUR HUMIDE
! VDHNS : VITESSE DE DEPOT DE L'OZONE EN HIVER POUR UNE NUIT SECHE
! VDHNH : VITESSE DE DEPOT DE L'OZONE EN HIVER POUR UNE NUIT HUMIDE
! VDPJS : VITESSE DE DEPOT DE L'OZONE AU PRINTEMPS POUR UN JOUR SEC
! VDPJH : VITESSE DE DEPOT DE L'OZONE AU PRINTEMPS POUR UN JOUR HUMIDE
! VDPNS : VITESSE DE DEPOT DE L'OZONE AU PRINTEMPS POUR UNE NUIT SECHE
! VDPNH : VITESSE DE DEPOT DE L'OZONE AU PRINTEMPS POUR UNE NUIT HUMIDE
! VDEJS : VITESSE DE DEPOT DE L'OZONE EN ETE POUR UN JOUR SEC
! VDEJH : VITESSE DE DEPOT DE L'OZONE EN ETE POUR UN JOUR HUMIDE
! VDENS : VITESSE DE DEPOT DE L'OZONE EN ETE POUR UNE NUIT SECHE
! VDENH : VITESSE DE DEPOT DE L'OZONE EN ETE POUR UNE NUIT HUMIDE
! VDAJS : VITESSE DE DEPOT DE L'OZONE EN AUTOMNE POUR UN JOUR SEC
! VDAJH : VITESSE DE DEPOT DE L'OZONE EN AUTOMNE POUR UN JOUR HUMIDE
! VDANS : VITESSE DE DEPOT DE L'OZONE EN AUTOMNE POUR UNE NUIT SECHE
! VDANH : VITESSE DE DEPOT DE L'OZONE EN AUTOMNE POUR UNE NUIT HUMIDE
! VDNJS : VITESSE DE DEPOT DE L'OZONE EN HIVER-NEIGE POUR UN JOUR SEC
! VDNJH : VITESSE DE DEPOT DE L'OZONE EN HIVER-NEIGE POUR UN JOUR HUMIDE 
! VDNNS : VITESSE DE DEPOT DE L'OZONE EN HIVER-NEIGE POUR UNE NUIT SECHE
! VDNNH : VITESSE DE DEPOT DE L'OZONE EN HIVER-NEIGE POUR UNE NUIT HUMIDE

! VOZNJ : SEUIL NUIT/JOUR  (EN W/M2 : SEUIL DU FLUX SOLAIRE A NFLEVG)
! VOZHS : SEUIL HUMIDE/SEC (EN kg/m2/s : SEUIL DE LA PLUIE TOTALE) 

! LRDIFOZ : POUR ACTIVER LE SCHEMA DE DIFUSION SEULE DE L'OZONE
! LRDEPOZ : POUR ACTIVER LE SCHEMA DE DIFFUSION+DEPOSITION DE L'OZONE

!  DECLARATION DE LA TAILLE DES TABLEAUX
!  CETTE TAILLE A ETE ELARGIE, EN FAIT LA DIMENSION DES TABLEAUX
!  EST LE NOMBRE DE CLASSES DE VEGETATION.

REAL(KIND=JPRB) :: VDHJS(99)
REAL(KIND=JPRB) :: VDHJH(99)
REAL(KIND=JPRB) :: VDHNS(99)
REAL(KIND=JPRB) :: VDHNH(99)
REAL(KIND=JPRB) :: VDPJS(99)
REAL(KIND=JPRB) :: VDPJH(99)
REAL(KIND=JPRB) :: VDPNS(99)
REAL(KIND=JPRB) :: VDPNH(99)
REAL(KIND=JPRB) :: VDEJS(99)
REAL(KIND=JPRB) :: VDEJH(99)
REAL(KIND=JPRB) :: VDENS(99)
REAL(KIND=JPRB) :: VDENH(99)
REAL(KIND=JPRB) :: VDAJS(99)
REAL(KIND=JPRB) :: VDAJH(99)
REAL(KIND=JPRB) :: VDANS(99)
REAL(KIND=JPRB) :: VDANH(99)
REAL(KIND=JPRB) :: VDNJS(99)
REAL(KIND=JPRB) :: VDNJH(99)
REAL(KIND=JPRB) :: VDNNS(99)
REAL(KIND=JPRB) :: VDNNH(99)
REAL(KIND=JPRB) :: VOZNJ
REAL(KIND=JPRB) :: VOZHS
LOGICAL :: LRDIFOZ
LOGICAL :: LRDEPOZ
!   ---------------------------------------------------------------------
!$OMP THREADPRIVATE(lrdepoz,lrdifoz,vdajh,vdajs,vdanh,vdans,vdejh,vdejs,vdenh,vdens,vdhjh,vdhjs)
!$OMP THREADPRIVATE(vdhnh,vdhns,vdnjh,vdnjs,vdnnh,vdnns,vdpjh,vdpjs,vdpnh,vdpns,vozhs,voznj)
END MODULE YOMVDOZ
