MODULE regdim_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: i1_deb, i1_fin, i2_deb, i2_fin, j_deb, j_fin

  INTEGER i1_deb, i1_fin
  INTEGER i2_deb, i2_fin

  PARAMETER (i1_deb = 16, i1_fin = 30)
  PARAMETER (i2_deb = 31, i2_fin = 33)

  INTEGER j_deb, j_fin

  PARAMETER (j_deb = 18, j_fin = 39)
END MODULE regdim_mod_h
