INTERFACE
SUBROUTINE LWB&
 & ( KIDIA, KFDIA, KLON , KLEV , KMODE,&
 & PDT0 , PTAVE, PTL,&
 & PB , PBINT, PBSUR , PBTOP , PDBSL,&
 & PGA , PGB , PGASUR, PGBSUR, PGATOP, PGBTOP&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOELW , ONLY : MXIXT ,NSIL ,NIPD ,PDGA ,&
 & PDGB ,TINTP ,TSTAND ,TSTP ,XP 
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KMODE
REAL(KIND=JPRB) ,INTENT(IN) :: PDT0(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAVE(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PTL(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PB(KLON,NSIL,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PBINT(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PBSUR(KLON,NSIL)
REAL(KIND=JPRB) ,INTENT(OUT) :: PBTOP(KLON,NSIL)
REAL(KIND=JPRB) ,INTENT(OUT) :: PDBSL(KLON,NSIL,KLEV*2)
REAL(KIND=JPRB) ,INTENT(OUT) :: PGA(KLON,NIPD,2,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PGB(KLON,NIPD,2,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PGASUR(KLON,NIPD,2)
REAL(KIND=JPRB) ,INTENT(OUT) :: PGBSUR(KLON,NIPD,2)
REAL(KIND=JPRB) ,INTENT(OUT) :: PGATOP(KLON,NIPD,2)
REAL(KIND=JPRB) ,INTENT(OUT) :: PGBTOP(KLON,NIPD,2)
END SUBROUTINE LWB
END INTERFACE
