! Replaces compbl.h

MODULE compbl_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC iflag_pbl, iflag_pbl_split, iflag_order2_sollw, ifl_pbltree, iflag_hetero_surf

  !!      integer iflag_pbl,iflag_pbl_split
  !!      common/compbl/iflag_pbl,iflag_pbl_split
  !!FC      integer iflag_pbl, iflag_pbl_split, iflag_order2_sollw
  !FC      common/compbl/iflag_pbl, iflag_pbl_split, iflag_order2_sollw
  INTEGER iflag_pbl, iflag_pbl_split, iflag_order2_sollw, ifl_pbltree, iflag_hetero_surf
  !$OMP THREADPRIVATE(iflag_pbl, iflag_pbl_split, iflag_order2_sollw, ifl_pbltree, iflag_hetero_surf)

  !>jyg+al1

END MODULE compbl_mod_h
