MODULE planete_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: aphelie, periheli, year_day, peri_day, obliquit, timeperi, e_elips, p_elips, unitastr

  REAL aphelie, periheli, year_day, peri_day, obliquit, timeperi, e_elips, &
          p_elips, unitastr

  !$OMP THREADPRIVATE(aphelie, periheli, year_day, peri_day, obliquit, timeperi, e_elips, p_elips, unitastr)
END MODULE planete_mod_h
