MODULE date_cas_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC year_ini_cas, mth_ini_cas, day_deb, heure_ini_cas, pdt_cas, day_ju_ini_cas

  INTEGER :: year_ini_cas    ! initial year of the case
  INTEGER :: mth_ini_cas     ! initial month of the case
  INTEGER :: day_deb         ! initial day of the case
  REAL :: heure_ini_cas      ! start time of the case
  REAL :: pdt_cas            ! forcing_frequency
  REAL :: day_ju_ini_cas     ! julian day of initial day of the case
END MODULE date_cas_mod_h

