INTERFACE
SUBROUTINE SWUVO3&
 & ( KIDIA,KFDIA,KLON,KNU,KABS,&
 & PU, PTR&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOMJFH , ONLY : N_VMASS
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KABS
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KNU
REAL(KIND=JPRB) ,INTENT(IN) :: PU(KLON,KABS)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTR(KLON,KABS)
END SUBROUTINE SWUVO3
END INTERFACE
