   subroutine call_param_replay(ngrid,nlay)
   integer,intent(in) :: ngrid,nlay
   stop 'In call_param_replay : You should run replay_equip.sh before runing replay[13]d'
   return
   end subroutine call_param_replay
