MODULE YOMCT0B

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!     ------------------------------------------------------------------

!*  Control variables for the default set-up DO NOT USE THEM OUT OF THE SET-UP !ub

! LECMWF  : .T. = set-up default values used at ECMWF
!           .F. = set-up default values used at METEO-FRANCE
LOGICAL :: LECMWF

!     ------------------------------------------------------------------
!$OMP THREADPRIVATE(lecmwf)
END MODULE YOMCT0B
