INTERFACE
SUBROUTINE SRTM_REFTRA&
 & ( KLEV , KMODTS,&
 & LDRTCHK,&
 & PGG , PRMUZ, PTAU , PW,&
 & PREF , PREFD, PTRA , PTRAD&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE PARSRTM , ONLY : JPLAY
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(OUT) :: KMODTS
LOGICAL ,INTENT(IN) :: LDRTCHK(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: PGG(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: PRMUZ
REAL(KIND=JPRB) ,INTENT(IN) :: PTAU(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: PW(JPLAY)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PREF(JPLAY)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PREFD(JPLAY)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PTRA(JPLAY)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PTRAD(JPLAY)
END SUBROUTINE SRTM_REFTRA
END INTERFACE
