INTERFACE
SUBROUTINE SUAERV15 ( KLEV , PETAH,&
 & PVDAES,PVDAEL,PVDAEU,PVDAED,PVDAEF,&
 & PTRBGA,PVOBGA,PSTBGA,PAEOPS,PAEOPL,PAEOPU,&
 & PAEOPF,&
 & PAEOPD,PTRPT,PAEADK,PAEADM, PAEROS ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
REAL(KIND=JPRB) ,INTENT(IN) :: PETAH(KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PVDAES(KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PVDAEL(KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PVDAEU(KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PVDAED(KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PVDAEF(KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTRBGA
REAL(KIND=JPRB) ,INTENT(OUT) :: PVOBGA
REAL(KIND=JPRB) ,INTENT(OUT) :: PSTBGA
REAL(KIND=JPRB) ,INTENT(OUT) :: PAEOPS
REAL(KIND=JPRB) ,INTENT(OUT) :: PAEOPL
REAL(KIND=JPRB) ,INTENT(OUT) :: PAEOPU
REAL(KIND=JPRB) ,INTENT(OUT) :: PAEOPF
REAL(KIND=JPRB) ,INTENT(OUT) :: PAEOPD
REAL(KIND=JPRB) ,INTENT(OUT) :: PTRPT
REAL(KIND=JPRB) ,INTENT(OUT) :: PAEADK(3)
REAL(KIND=JPRB) ,INTENT(OUT) :: PAEADM
REAL(KIND=JPRB) ,INTENT(OUT) :: PAEROS
END SUBROUTINE SUAERV15
END INTERFACE
