INTERFACE 
SUBROUTINE RRTM_ECRT_140GP &
 & ( K_IPLON, klon , klev, kcld,&
 & paer , paph , pap,&
 & pts  , pth  , pt,&
 & P_ZEMIS, P_ZEMIW,&
 & pq   , pcco2, pozn, pcldf, ptaucld, ptclear,&
 & P_CLDFRAC,P_TAUCLD,&
 & PTAU_LW,&
 & P_COLDRY,P_WKL,P_WX,&
 & P_TAUAERL,PAVEL,P_TAVEL,PZ,P_TZ,P_TBOUND,K_NLAYERS,P_SEMISS,K_IREFLECT )
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE PARRRTM  , ONLY : JPBAND   ,JPXSEC   ,JPLAY   ,&
 & JPINPX  
USE YOERAD   , ONLY : NLW      ,NOVLP
USE YOERDI   , ONLY :    RCH4     ,RN2O    ,RCFC11  ,RCFC12
USE YOESW    , ONLY : RAER
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: K_IPLON
INTEGER(KIND=JPIM),INTENT(OUT) :: KCLD
REAL(KIND=JPRB) ,INTENT(IN) :: PAER(KLON,6,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PAPH(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PAP(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PTS(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PTH(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PT(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: P_ZEMIS(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: P_ZEMIW(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PQ(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PCCO2
REAL(KIND=JPRB) ,INTENT(IN) :: POZN(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PCLDF(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAUCLD(KLON,KLEV,JPBAND)
!--C.Kleinschmitt
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAU_LW(KLON,KLEV,NLW) ! LW Optical depth of aerosols  
!--end
REAL(KIND=JPRB) ,INTENT(OUT) :: PTCLEAR
REAL(KIND=JPRB) ,INTENT(OUT) :: P_CLDFRAC(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_TAUCLD(JPLAY,JPBAND)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_COLDRY(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_WKL(JPINPX,JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_WX(JPXSEC,JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_TAUAERL(JPLAY,JPBAND)
REAL(KIND=JPRB) ,INTENT(OUT) :: PAVEL(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_TAVEL(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PZ(0:JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_TZ(0:JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_TBOUND
INTEGER(KIND=JPIM),INTENT(OUT) :: K_NLAYERS
REAL(KIND=JPRB) ,INTENT(OUT) :: P_SEMISS(JPBAND)
INTEGER(KIND=JPIM),INTENT(OUT) :: K_IREFLECT
END SUBROUTINE RRTM_ECRT_140GP
END INTERFACE
