module therm_expans_m

  implicit none

contains

  elemental real function therm_expans(t)

    ! Compute the thermal expansion coefficient of sea water.

    real, intent(in):: t ! temperature, in K

    !--------------------------------------------------------------

    therm_expans = 2.1e-5 * (t - 269.95)**0.79

  end function therm_expans

end module therm_expans_m
