INTERFACE
SUBROUTINE RRTM_RTRN1A_140GP (KLEV,K_ISTART,K_IEND,K_ICLDLYR,P_CLDFRAC,P_TAUCLD,P_ABSS1,&
 & P_OD,P_TAUSF1,P_CLFNET,P_CLHTR,P_FNET,P_HTR,P_TOTDFLUC,P_TOTDFLUX,P_TOTUFLUC,P_TOTUFLUX,&
 & P_TOAG, P_TOACG, &
 & P_TAVEL,PZ,P_TZ,P_TBOUND,PFRAC,P_SEMISS,P_SEMISLW,K_IREFLECT) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE PARRRTM , ONLY : JPBAND ,JPGPT ,JPLAY
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: K_ISTART
INTEGER(KIND=JPIM),INTENT(IN) :: K_IEND
INTEGER(KIND=JPIM),INTENT(IN) :: K_ICLDLYR(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_CLDFRAC(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_TAUCLD(JPLAY,JPBAND)
REAL(KIND=JPRB) ,INTENT(IN) :: P_ABSS1(JPGPT*JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_OD(JPGPT,JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_TAUSF1(JPGPT*JPLAY)
REAL(KIND=JPRB) :: P_CLFNET(0:JPLAY)
REAL(KIND=JPRB) :: P_CLHTR(0:JPLAY)
REAL(KIND=JPRB) :: P_FNET(0:JPLAY)
REAL(KIND=JPRB) :: P_HTR(0:JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_TOTDFLUC(0:JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_TOTDFLUX(0:JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_TOTUFLUC(0:JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_TOTUFLUX(0:JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_TOAG(JPGPT) !FC
REAL(KIND=JPRB) ,INTENT(OUT) :: P_TOACG(JPGPT) !FC
REAL(KIND=JPRB) ,INTENT(IN) :: P_TAVEL(JPLAY)
REAL(KIND=JPRB) :: PZ(0:JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_TZ(0:JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_TBOUND
REAL(KIND=JPRB) ,INTENT(IN) :: PFRAC(JPGPT,JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_SEMISS(JPBAND)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_SEMISLW
INTEGER(KIND=JPIM) :: K_IREFLECT
END SUBROUTINE RRTM_RTRN1A_140GP
END INTERFACE
