INTERFACE
SUBROUTINE RRTM_GASABS1A_140GP (KLEV,P_ATR1,P_OD,P_TF1,P_COLDRY,P_WX,&
 & P_TAUAERL,P_FAC00,P_FAC01,P_FAC10,P_FAC11,P_FORFAC,K_JP,K_JT,K_JT1,P_ONEMINUS,&
 & P_COLH2O,P_COLCO2,P_COLO3,P_COLN2O,P_COLCH4,P_COLO2,P_CO2MULT,&
 & K_LAYTROP,K_LAYSWTCH,K_LAYLOW,P_SELFFAC,P_SELFFRAC,K_INDSELF,PFRAC) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE PARRRTM , ONLY : JPLAY ,JPBAND ,JPGPT ,JPXSEC
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
REAL(KIND=JPRB) ,INTENT(OUT) :: P_ATR1(JPGPT,JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_OD(JPGPT,JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_TF1(JPGPT,JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_COLDRY(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_WX(JPXSEC,JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_TAUAERL(JPLAY,JPBAND)
REAL(KIND=JPRB) ,INTENT(IN) :: P_FAC00(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_FAC01(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_FAC10(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_FAC11(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_FORFAC(JPLAY)
INTEGER(KIND=JPIM),INTENT(IN) :: K_JP(JPLAY)
INTEGER(KIND=JPIM),INTENT(IN) :: K_JT(JPLAY)
INTEGER(KIND=JPIM),INTENT(IN) :: K_JT1(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_ONEMINUS
REAL(KIND=JPRB) ,INTENT(IN) :: P_COLH2O(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_COLCO2(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_COLO3(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_COLN2O(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_COLCH4(JPLAY)
REAL(KIND=JPRB) :: P_COLO2(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_CO2MULT(JPLAY)
INTEGER(KIND=JPIM),INTENT(IN) :: K_LAYTROP
INTEGER(KIND=JPIM),INTENT(IN) :: K_LAYSWTCH
INTEGER(KIND=JPIM),INTENT(IN) :: K_LAYLOW
REAL(KIND=JPRB) ,INTENT(IN) :: P_SELFFAC(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_SELFFRAC(JPLAY)
INTEGER(KIND=JPIM),INTENT(IN) :: K_INDSELF(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFRAC(JPGPT,JPLAY)
END SUBROUTINE RRTM_GASABS1A_140GP
END INTERFACE
