!     -----------------------------------------------------------------
NAMELIST/NAEPHY/&
       &LEPHYS, LECOND, LECUMF, LEDCLD, LEEVAP &
 &,     LEGWDG, LEOZOC, LEQNGT, LERADI, LERADS &
 &,     LESHCV, LESICE, LESURF, LEVDIF, LE4ALB &
 &,     LEPCLD, LAGPHY ,LEO3CH, LECO2DIU &
 &,     LWCOU  ,LWCOU2W, LBUD23,LEMETHOX,LERA40,LERAIN &
 &,     LVDFTRAC,LMFTRAC,LEOCWA,LEOCCO,LEOCSA,NSTPW &
 &,     LECURR,RSOUTW ,RNORTW ,RDEGREW &
 &,     RTHRFRTI &
!&,     RCLDTOPP &      MPL 24.11.08
!&,     NPHYINT &       MPL 24.11.08
 &,     NEPHYS_PCFULL &
 &,     LEMWAVE
!     -----------------------------------------------------------------


