INTERFACE
SUBROUTINE SWDE&
 & ( KIDIA, KFDIA, KLON,&
 & PGG , PREF , PRMUZ, PTO1, PW,&
 & PRE1 , PRE2 , PTR1 , PTR2&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOMJFH , ONLY : N_VMASS
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
REAL(KIND=JPRB) ,INTENT(IN) :: PGG(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PREF(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PRMUZ(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PTO1(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PW(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRE1(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRE2(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTR1(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTR2(KLON)
END SUBROUTINE SWDE
END INTERFACE
