INTERFACE
SUBROUTINE SUGWD(KULOUT,KLEV,PVAH,PVBH)
USE PARKIND1 ,ONLY : JPIM ,JPRB
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KULOUT
REAL(KIND=JPRB) ,INTENT(IN) :: PVAH(KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PVBH(KLEV+1)
END SUBROUTINE SUGWD
END INTERFACE
