MODULE chem_mod_h

! AS 20250220 : masse_s MOVED TO Dust/lmdz_spla_ini.f90 
!               n_avogadro, masse_so4 REMOVED BECAUSE NOT USED      

  IMPLICIT NONE; PRIVATE
  PUBLIC :: idms, iso2, iso4, ih2s, idmso, imsa, ih2o2, &
           rho_water, rho_ice

  INTEGER idms, iso2, iso4, ih2s, idmso, imsa, ih2o2
  PARAMETER (idms = 1, iso2 = 2, iso4 = 3)
  PARAMETER (ih2s = 4, idmso = 5, imsa = 6, ih2o2 = 7)

  REAL rho_water, rho_ice
  PARAMETER (rho_water = 1000.0)                    !--kg m-3
  PARAMETER (rho_ice = 500.0)                       !--kg m-3

END MODULE chem_mod_h

