MODULE thermcell_old_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: r_aspect_thermals, l_mix_thermals, w2di_thermals
  real, parameter :: r_aspect_thermals = 2.
  real, parameter :: l_mix_thermals = 30.
  integer, parameter :: w2di_thermals = 0
END MODULE thermcell_old_mod_h
