INTERFACE
SUBROUTINE SUCLOPN (KTSW, KSW, KLEV)
USE PARKIND1 ,ONLY : JPIM ,JPRB
INTEGER(KIND=JPIM),INTENT(IN) :: KTSW
INTEGER(KIND=JPIM),INTENT(IN) :: KSW
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
END SUBROUTINE SUCLOPN
END INTERFACE
