INTERFACE
SUBROUTINE SWU&
 & ( KIDIA, KFDIA , KLON , KLEV,&
 & PSCT , PCARDI, PCLDSW, PPMB , PPSOL, PRMU0, PTAVE, PWV,&
 & PAKI , PCLD , PCLEAR, PDSIG, PFACT, PRMU , PSEC , PUD&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOERAD , ONLY : NOVLP
! Temporary fix waiting for cleaner interface (or not)
USE clesphys_mod_h, ONLY: NSW
!!include "clesphys.h"
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
REAL(KIND=JPRB) ,INTENT(IN) :: PSCT
REAL(KIND=JPRB) ,INTENT(IN) :: PCARDI
REAL(KIND=JPRB) ,INTENT(IN) :: PCLDSW(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PPMB(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PPSOL(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PRMU0(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAVE(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PWV(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PAKI(KLON,2,NSW)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PCLD(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PCLEAR(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PDSIG(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFACT(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRMU(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PSEC(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PUD(KLON,5,KLEV+1)
END SUBROUTINE SWU
END INTERFACE
