INTERFACE
SUBROUTINE SRTM_CLDPROP&
 & ( KLEV, K_ICLDATM, K_INFLAG, K_ICEFLAG, K_LIQFLAG, K_NSTR,&
 & P_CLDFRAC, P_CLDDAT1, P_CLDDAT2, P_CLDDAT3, P_CLDDAT4, P_CLDDATMOM,&
 & P_TAUCLDORIG, P_TAUCLOUD, P_SSACLOUD, P_XMOM&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE PARSRTM , ONLY : JPLAY, JPBAND, JPB1, JPB2
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(OUT) :: K_ICLDATM
INTEGER(KIND=JPIM),INTENT(IN) :: K_INFLAG
INTEGER(KIND=JPIM),INTENT(IN) :: K_ICEFLAG
INTEGER(KIND=JPIM),INTENT(IN) :: K_LIQFLAG
INTEGER(KIND=JPIM),INTENT(IN) :: K_NSTR
REAL(KIND=JPRB) ,INTENT(IN) :: P_CLDFRAC(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_CLDDAT1(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_CLDDAT2(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_CLDDAT3(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_CLDDAT4(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: P_CLDDATMOM(0:16,JPLAY)
REAL(KIND=JPRB) ,INTENT(INOUT) :: P_TAUCLDORIG(JPLAY,JPBAND)
REAL(KIND=JPRB) ,INTENT(INOUT) :: P_TAUCLOUD(JPLAY,JPBAND)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_SSACLOUD(JPLAY,JPBAND)
REAL(KIND=JPRB) ,INTENT(OUT) :: P_XMOM(0:16,JPLAY,JPBAND)
END SUBROUTINE SRTM_CLDPROP
END INTERFACE
