! Replaces comsoil.h

MODULE comsoil_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: inertie_sol, inertie_sno, inertie_sic, inertie_lic, iflag_sic, iflag_inertie

  REAL inertie_sol, inertie_sno, inertie_sic, inertie_lic
  INTEGER iflag_sic, iflag_inertie
  !$OMP THREADPRIVATE(inertie_sol,inertie_sno,inertie_sic,inertie_lic, iflag_sic,iflag_inertie)
END MODULE comsoil_mod_h
