!     ------------------------------------------------------------------
NAMELIST/NAMPHY/&
! * GROUPE DES CLES GENERALES.
  &LMPHYS,LSLC &
! * GROUPE DES CLES NE COMMANDANT PAS L'APPEL AUX ROUTINES AM-----.
  &,LCAPE,LCONDWT,LCVCAS,LCVDD,LCVLIS,LCVPP,LHUNEG,LNEIGE &
  &,LRNUMX,LCLSATUR,LSRCON,LSRCONT,L2PHYS,LRRGUST,LO3ABC,LNEBNXR &
  &,LFPCOR,LNOIAS,LGLT,LNEWD,LHUCN,LQXRTGH &
  &,LPROCLD,LEVAPP,LCOLLEC,LNSMLIS,LRELAXT,LRELAXW &
  &,LAEROSEA,LAEROLAN,LAEROSOO,LAERODES,LAEROVOL,LAEROSUL &
  &,LSSD,LPHCDPI,LBLVAR &
  &,LZ0HSREL,LCVPRO,LCDDPRO &

! * GROUPE DES CLES COMMANDANT L'APPEL AUX ROUTINES AM-----.
  &,CGMIXLEN,LPRGML,LCVRA,LGWD,LGWDC,LNEBCO,LNEBN,LNEBR,LNEBT,LND2DIFF &
  &,LOZONE,LRAY,LRAYLU,LREWS &
  &,LRPROX,LRMIX,LRSTAB,LRAUTOEV,LVOIGT,LVFULL,LRTPP,LRTDL,LRAYPL &
  &,LRAYFM,LRAYFM15,LRRMES,LSFHYD,LSNV,LSOLV,LFGEL,LSTRA,LSTRAS &
  &,LVDIF,L3MT,LSCMF &
! * GROUPE DES INDICES DE CALCUL.
  &,NBITER,NOIR,NDPSFI,NPHYREP,LVGSN,LDIFCONS,LECT,LPTKE,LCVPGY,LPBLE &
  &,LADJCLD,LCVPPKF,LECTFL,LAJUCV,LNEBGR,LNEBGY,LCVRAV3,LBCCOND &
  &,LPIL, LSTRAPRO, LNEWSTAT, LPHSPSH, LSMNIMBT, LSMROT, LSMTPS &
  &,L1DRHCRI, LGWRHCRI, NSMTBOT, NSMDNEB, NPRAG, NPRAC, NPRRI &
  &,LAUTONEB
!     ------------------------------------------------------------------
