INTERFACE
SUBROUTINE LWC&
 & ( KIDIA , KFDIA, KLON , KLEV,&
 & PBINT , PBSUI, PCLDLD, PCLDLU,&
 & PCNTRB, PEMIT, PFLUC,&
 & PFLUX&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
REAL(KIND=JPRB) ,INTENT(IN) :: PBINT(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PBSUI(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PCLDLD(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PCLDLU(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PCNTRB(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PEMIT(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PFLUC(KLON,2,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFLUX(KLON,2,KLEV+1)
END SUBROUTINE LWC
END INTERFACE
