MODULE dimsoil_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: nsoilmx, nsnowmx, nsismx

  INTEGER nsoilmx
  PARAMETER (nsoilmx = 11)

  ! For Inlandsis, Etienne Vignon:
  INTEGER nsnowmx
  PARAMETER (nsnowmx = 30)

  INTEGER nsismx
  PARAMETER (nsismx = 41)
  ! nsismx should be equal to nsoilmx+nsnowmx
END MODULE dimsoil_mod_h






