!     -----------------------------------------------------------------
NAMELIST/NAEAER/&
  & LEPAERO  , LAERCLIMG, LAERCLIMZ, LAERCLIST, LAERDRYDP, LAEREXTR , &
  & LAERGBUD , LAERNGAT , LAERPRNT , LAERSCAV , LAERSEDIM, LAERSURF , &
  & LAER6SDIA, &
  & NTAER    , NTYPAER  , INDBG    , NDDUST   , &
  & RLATVOL  , RLONVOL  , KSTPDBG
!     -----------------------------------------------------------------

