INTERFACE
SUBROUTINE LWTT ( KIDIA, KFDIA, KLON, PGA , PGB, PUU , PTT )
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOELW , ONLY : NTRA ,NUA ,RPTYPE ,RETYPE ,&
 & RO1H ,RO2H ,RPIALF0 
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
REAL(KIND=JPRB) ,INTENT(IN) :: PGA(KLON,8,2)
REAL(KIND=JPRB) ,INTENT(IN) :: PGB(KLON,8,2)
REAL(KIND=JPRB) ,INTENT(IN) :: PUU(KLON,NUA)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTT(KLON,NTRA)
END SUBROUTINE LWTT
END INTERFACE
