INTERFACE
SUBROUTINE SRTM_SRTM_224GP_MCICA&
 & ( KIDIA , KFDIA , KLON , KLEV , KSW , KCOLS , KCLDLY ,&
 & PAER , PALBD , PALBP , PAPH , PAP ,&
 & PTS , PTH , PT ,&
 & PQ , PCCO2 , POZN , PRMU0 ,&
 & PFRCL , PTAUC , PASYC , POMGC ,&
 & PFSUX , PFSUC&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE PARSRTM , ONLY : JPLAY
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KSW
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KCOLS
INTEGER(KIND=JPIM),INTENT(IN) :: KCLDLY(KCOLS)
REAL(KIND=JPRB) ,INTENT(IN) :: PAER(KLON,6,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PALBD(KLON,KSW)
REAL(KIND=JPRB) ,INTENT(IN) :: PALBP(KLON,KSW)
REAL(KIND=JPRB) ,INTENT(IN) :: PAPH(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PAP(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PTS(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PTH(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PT(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PQ(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PCCO2
REAL(KIND=JPRB) ,INTENT(IN) :: POZN(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PRMU0(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PFRCL(KLON,KCOLS,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAUC(KLON,KCOLS,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PASYC(KLON,KCOLS,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: POMGC(KLON,KCOLS,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFSUX(KLON,2,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFSUC(KLON,2,KLEV+1)
END SUBROUTINE SRTM_SRTM_224GP_MCICA
END INTERFACE
