INTERFACE
SUBROUTINE LWBV&
 & ( KIDIA, KFDIA, KLON , KLEV , KMODE,&
 & PDT0 , PEMIS, PEMIW,&
 & PTL , PTAVE,&
 & PEMIT, PFLUC,&
 & PABCU, PBINT, PBSUI, PCNTRB&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOELW , ONLY : NSIL ,NIPD ,NUA
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KMODE
REAL(KIND=JPRB) ,INTENT(IN) :: PDT0(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PEMIS(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PEMIW(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PTL(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAVE(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PEMIT(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFLUC(KLON,2,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PABCU(KLON,NUA,3*KLEV+1)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PBINT(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PBSUI(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PCNTRB(KLON,KLEV+1,KLEV+1)
END SUBROUTINE LWBV
END INTERFACE
