! Replaces flux_arh.h
! Modif EV, 10/2020

MODULE flux_arp_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: fsens, flat, betaevap, ust, tg, ok_flux_surf, ok_prescr_ust, ok_prescr_beta, ok_forc_tsurf

  LOGICAL :: ok_flux_surf
  LOGICAL :: ok_prescr_ust !for prescribed ustar
  LOGICAL :: ok_prescr_beta
  LOGICAL :: ok_forc_tsurf

  REAL :: fsens
  REAL :: flat
  REAL :: betaevap
  REAL :: ust
  REAL :: tg

  !$OMP THREADPRIVATE(fsens,flat,betaevap,ust,tg,ok_flux_surf,ok_prescr_ust,ok_prescr_beta,ok_forc_tsurf)
END MODULE flux_arp_mod_h


      

