!  ---------------------------------------------------------------
NAMELIST/NAMVDOZ/&
     &VDHJS,VDHJH,VDHNS,VDHNH &
    &,VDPJS,VDPJH,VDPNS,VDPNH &
    &,VDEJS,VDEJH,VDENS,VDENH &
    &,VDAJS,VDAJH,VDANS,VDANH &
    &,VDNJS,VDNJH,VDNNS,VDNNH &
    &,VOZNJ,VOZHS &
    &,LRDIFOZ,LRDEPOZ


