!
! $Id: pbl_surface_mod.F90 6014 2026-01-12 14:20:38Z ymeurdesoif $
!
MODULE pbl_surface_mod
!
! Planetary Boundary Layer and Surface module
!
! This module manages the calculation of turbulent diffusion in the boundary layer 
! and all interactions towards the differents sub-surfaces.
!
!
  USE pbl_surface_data, ONLY : ftsoil, ydTs0, ydqs0, snow, iflag_pbl_surface_t2m_bug, iflag_new_t2mq2m, & 
                               ok_bug_zg_wk_pbl, smallestreal, beta_land
  USE pbl_surface_main_mod

END MODULE pbl_surface_mod
