INTERFACE
SUBROUTINE LW&
 & ( KIDIA, KFDIA , KLON , KLEV , KMODE,&
 & PCCO2, PCLDLD, PCLDLU,&
 & PDP , PDT0 , PEMIS , PEMIW,&
 & PPMB , PQOF , PTL,&
 & PAER , PTAVE , PVIEW , PWV,&
 & PEMIT, PFLUX , PFLUC&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOELW , ONLY : NUA
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KMODE
REAL(KIND=JPRB) ,INTENT(IN) :: PCCO2
REAL(KIND=JPRB) ,INTENT(IN) :: PCLDLD(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PCLDLU(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PDP(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PDT0(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PEMIS(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PEMIW(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PPMB(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PQOF(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PTL(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PAER(KLON,6,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAVE(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PVIEW(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PWV(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PEMIT(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFLUX(KLON,2,KLEV+1)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PFLUC(KLON,2,KLEV+1)
END SUBROUTINE LW
END INTERFACE
