MODULE PARSRTM

USE PARKIND1  ,ONLY : JPIM

IMPLICIT NONE

SAVE

!     ------------------------------------------------------------------
!     Parameters relevant to AER's RRTM-SW radiation scheme

!     030224  JJMorcrette

!     Modified for g-point reduction from 224 to 112.  
!     Swap code below to restore 224 g-point set. 
!     Mar2004 MJIacono, AER
!     ------------------------------------------------------------------

INTEGER(KIND=JPIM), PARAMETER :: JPLAY  = 100

INTEGER(KIND=JPIM), PARAMETER :: JPG    = 16
INTEGER(KIND=JPIM), PARAMETER :: JPBAND = 29
INTEGER(KIND=JPIM), PARAMETER :: JPSW   = 14
INTEGER(KIND=JPIM), PARAMETER :: JPB1   = 16
INTEGER(KIND=JPIM), PARAMETER :: JPB2   = 29
! Use for 112 g-points
INTEGER(KIND=JPIM), PARAMETER :: JPGPT  = 112
! Use for 224 g-points
!INTEGER(KIND=JPIM), PARAMETER :: JPGPT  = 224

INTEGER(KIND=JPIM), PARAMETER :: JMCMU  = 32
INTEGER(KIND=JPIM), PARAMETER :: JMUMU  = 32
INTEGER(KIND=JPIM), PARAMETER :: JMPHI  = 3
INTEGER(KIND=JPIM), PARAMETER :: JMXANG = 4
INTEGER(KIND=JPIM), PARAMETER :: JMXSTR = 16

! Use for 112 g-points
INTEGER(KIND=JPIM), PARAMETER :: NG16 = 6
INTEGER(KIND=JPIM), PARAMETER :: NG17 = 12
INTEGER(KIND=JPIM), PARAMETER :: NG18 = 8
INTEGER(KIND=JPIM), PARAMETER :: NG19 = 8
INTEGER(KIND=JPIM), PARAMETER :: NG20 = 10
INTEGER(KIND=JPIM), PARAMETER :: NG21 = 10
INTEGER(KIND=JPIM), PARAMETER :: NG22 = 2
INTEGER(KIND=JPIM), PARAMETER :: NG23 = 10
INTEGER(KIND=JPIM), PARAMETER :: NG24 = 8
INTEGER(KIND=JPIM), PARAMETER :: NG25 = 6
INTEGER(KIND=JPIM), PARAMETER :: NG26 = 6
INTEGER(KIND=JPIM), PARAMETER :: NG27 = 8
INTEGER(KIND=JPIM), PARAMETER :: NG28 = 6
INTEGER(KIND=JPIM), PARAMETER :: NG29 = 12

! Use for 224 g-points
!INTEGER(KIND=JPIM), PARAMETER :: NG16 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG17 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG18 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG19 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG20 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG21 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG22 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG23 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG24 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG25 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG26 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG27 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG28 = 16
!INTEGER(KIND=JPIM), PARAMETER :: NG29 = 16

INTEGER(KIND=JPIM), PARAMETER :: NGS16 = 0
INTEGER(KIND=JPIM), PARAMETER :: NGS17 = 16
INTEGER(KIND=JPIM), PARAMETER :: NGS18 = 32
INTEGER(KIND=JPIM), PARAMETER :: NGS19 = 48
INTEGER(KIND=JPIM), PARAMETER :: NGS20 = 64
INTEGER(KIND=JPIM), PARAMETER :: NGS21 = 80
INTEGER(KIND=JPIM), PARAMETER :: NGS22 = 96
INTEGER(KIND=JPIM), PARAMETER :: NGS23 = 112
INTEGER(KIND=JPIM), PARAMETER :: NGS24 = 128
INTEGER(KIND=JPIM), PARAMETER :: NGS25 = 144
INTEGER(KIND=JPIM), PARAMETER :: NGS26 = 160
INTEGER(KIND=JPIM), PARAMETER :: NGS27 = 176
INTEGER(KIND=JPIM), PARAMETER :: NGS28 = 192
INTEGER(KIND=JPIM), PARAMETER :: NGS29 = 208

!     ------------------------------------------------------------------
END MODULE PARSRTM

