interface
SUBROUTINE LIQUID_EFFECTIVE_RADIUS &
 & (YDERAD,KIDIA, KFDIA, KLON, KLEV, &
 & PPRESSURE, PTEMPERATURE, PCLOUD_FRAC, PQ_LIQ, PQ_RAIN, &
 & PLAND_FRAC, PCCN_LAND, PCCN_SEA, &
 & PRE_UM)
USE PARKIND1 , ONLY : JPIM, JPRB
USE YOERAD , ONLY : TERAD
TYPE(TERAD) ,INTENT(IN):: YDERAD
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
REAL(KIND=JPRB), INTENT(IN) :: PPRESSURE(KLON,KLEV)
REAL(KIND=JPRB), INTENT(IN) :: PTEMPERATURE(KLON,KLEV)
REAL(KIND=JPRB), INTENT(IN) :: PCLOUD_FRAC(KLON,KLEV)
REAL(KIND=JPRB), INTENT(IN) :: PQ_LIQ(KLON,KLEV)
REAL(KIND=JPRB), INTENT(IN) :: PQ_RAIN(KLON,KLEV)
REAL(KIND=JPRB), INTENT(IN) :: PLAND_FRAC(KLON)
REAL(KIND=JPRB), INTENT(IN) :: PCCN_LAND(KLON)
REAL(KIND=JPRB), INTENT(IN) :: PCCN_SEA(KLON)
REAL(KIND=JPRB), INTENT(OUT) :: PRE_UM(KLON,KLEV)
END SUBROUTINE LIQUID_EFFECTIVE_RADIUS
end interface
