MODULE VAR_SV
      USE dimsoil_mod_h, ONLY: nsoilmx, nsnowmx
IMPLICIT NONE; PRIVATE
      PUBLIC nsol, nsot, nsno, nb_wri, klonv, knonv

! +
! +   LMDZ_SV dims
! +   ===================
! +
!      INTEGER,PARAMETER :: nsol=nsoilmx-1
!      INTEGER,PARAMETER :: nsno=nsnowmx
     
      INTEGER, PARAMETER    :: nsol=nsoilmx-1
      INTEGER, PARAMETER    :: nsot=nsoilmx+1
      INTEGER, PARAMETER    :: nsno=nsnowmx
      INTEGER, PARAMETER    :: nb_wri=1
      INTEGER,SAVE      :: klonv
!$OMP THREADPRIVATE(klonv)
      INTEGER,SAVE      :: knonv
!$OMP THREADPRIVATE(knonv)



END MODULE VAR_SV
