INTERFACE
SUBROUTINE SWCLR&
 & ( KIDIA , KFDIA , KLON , KLEV , KAER , KNU,&
 & PAER , PALBP , PDSIG , PRAYL , PSEC,&
 & PCGAZ , PPIZAZ, PRAY1 , PRAY2 , PREFZ , PRJ,&
 & PRK , PRMU0 , PTAUAZ, PTRA1 , PTRA2 , PTRCLR,&
 & LRDUST,PPIZA_DST, PCGA_DST, PTAUREL_DST ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOERAD , ONLY : NOVLP 
! Temporary fix waiting for cleaner interface (or not)
USE clesphys_mod_h, ONLY: NSW
!!include "clesphys.h"
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KAER
INTEGER(KIND=JPIM),INTENT(IN) :: KNU
REAL(KIND=JPRB) ,INTENT(IN) :: PAER(KLON,6,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PALBP(KLON,NSW)
REAL(KIND=JPRB) ,INTENT(IN) :: PDSIG(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PRAYL(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PSEC(KLON)
LOGICAL ,INTENT(IN) :: LRDUST
REAL(KIND=JPRB) ,INTENT(IN) :: PPIZA_DST(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PCGA_DST(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAUREL_DST(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PCGAZ(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PPIZAZ(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRAY1(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRAY2(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PREFZ(KLON,2,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRJ(KLON,6,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRK(KLON,6,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRMU0(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTAUAZ(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTRA1(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTRA2(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTRCLR(KLON)
END SUBROUTINE SWCLR
END INTERFACE
