INTERFACE
SUBROUTINE SRTM_SETCOEF&
 & ( KLEV , KNMOL ,&
 & PAVEL , PTAVEL , PZ , PTZ , PTBOUND ,&
 & PCOLDRY , PWKL ,&
 & KLAYTROP, KLAYSWTCH, KLAYLOW ,&
 & PCO2MULT, PCOLCH4 , PCOLCO2 , PCOLH2O , PCOLMOL , PCOLN2O , PCOLO2 , PCOLO3 ,&
 & PFORFAC , PFORFRAC , KINDFOR , PSELFFAC, PSELFFRAC, KINDSELF ,&
 & PFAC00 , PFAC01 , PFAC10 , PFAC11 ,&
 & KJP , KJT , KJT1&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE PARSRTM , ONLY : JPLAY
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM) :: KNMOL
REAL(KIND=JPRB) ,INTENT(IN) :: PAVEL(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAVEL(JPLAY)
REAL(KIND=JPRB) :: PZ(0:JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: PTZ(0:JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: PTBOUND
REAL(KIND=JPRB) ,INTENT(IN) :: PCOLDRY(JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: PWKL(35,JPLAY)
INTEGER(KIND=JPIM),INTENT(OUT) :: KLAYTROP
INTEGER(KIND=JPIM),INTENT(OUT) :: KLAYSWTCH
INTEGER(KIND=JPIM),INTENT(OUT) :: KLAYLOW
REAL(KIND=JPRB) ,INTENT(OUT) :: PCO2MULT(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PCOLCH4(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PCOLCO2(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PCOLH2O(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PCOLMOL(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PCOLN2O(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PCOLO2(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PCOLO3(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFORFAC(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFORFRAC(JPLAY)
INTEGER(KIND=JPIM),INTENT(OUT) :: KINDFOR(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PSELFFAC(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PSELFFRAC(JPLAY)
INTEGER(KIND=JPIM),INTENT(OUT) :: KINDSELF(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFAC00(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFAC01(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFAC10(JPLAY)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFAC11(JPLAY)
INTEGER(KIND=JPIM),INTENT(OUT) :: KJP(JPLAY)
INTEGER(KIND=JPIM),INTENT(OUT) :: KJT(JPLAY)
INTEGER(KIND=JPIM),INTENT(OUT) :: KJT1(JPLAY)
END SUBROUTINE SRTM_SETCOEF
END INTERFACE
