INTERFACE
SUBROUTINE SWR&
 & ( KIDIA , KFDIA , KLON , KLEV , KNU,&
 & PALBD , PCG , PCLD , POMEGA, PSEC , PTAU,&
 & PCGAZ , PPIZAZ, PRAY1, PRAY2 , PREFZ, PRJ , PRK , PRMUE,&
 & PTAUAZ, PTRA1 , PTRA2, PTRCLD&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOERAD , ONLY : NOVLP 
! Temporary fix waiting for cleaner interface (or not)
USE clesphys_mod_h, ONLY: NSW
!!include "clesphys.h"
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KNU
REAL(KIND=JPRB) ,INTENT(IN) :: PALBD(KLON,NSW)
REAL(KIND=JPRB) ,INTENT(IN) :: PCG(KLON,NSW,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PCLD(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: POMEGA(KLON,NSW,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PSEC(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAU(KLON,NSW,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PCGAZ(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PPIZAZ(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRAY1(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRAY2(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PREFZ(KLON,2,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRJ(KLON,6,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRK(KLON,6,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRMUE(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAUAZ(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTRA1(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTRA2(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTRCLD(KLON)
END SUBROUTINE SWR
END INTERFACE
