MODULE radopt_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: levoigt, novlp, kaer

  LOGICAL LEVOIGT
  PARAMETER (LEVOIGT = .FALSE.)
  INTEGER NOVLP
  PARAMETER (NOVLP = 1)
  INTEGER KAER
  PARAMETER (KAER = 0)
END MODULE radopt_mod_h
