
MODULE frequences_LW_data

    USE AERO_MOD, ONLY : nbands_lw_rrtm !FC
    IMPLICIT NONE
!FC
    REAL, SAVE, DIMENSION(nbands_lw_rrtm) :: wl1_lw , wl2_lw , deltanu   ! wavelenghts  bounds for  LW
!    REAL, SAVE, DIMENSION(nbands_lw_rrtm) :: wl1_lw , wl2_lw 
    !$OMP THREADPRIVATE(wl1_lw,wl2_lw,deltanu)


!      wl1_lw(1:nbands_lw_rrtm) = [ 10., 250., 500., 630., 700., 820., 980.,1080. ,1180.,1390.,1480.,1800.,2080.,2250.,2380.,2600.]
!      wl2_lw(1:nbands_lw_rrtm) = [250., 500., 630., 700., 820., 980.,1080.,1180. ,1390.,1480.,1800.,2080.,2250.,2380.,2600.,3000.]
!FC

!        DO ILW=1,nbands_lw_rrtm
!       deltanu(ILW)=(wl2_lw(ILW)-wl1_lw(ILW))/2.
!        print*, ILW,deltanu(ilw)
!       enddo
END MODULE frequences_LW_data

