INTERFACE
SUBROUTINE LWV&
 & ( KIDIA, KFDIA, KLON , KLEV , KUAER , KTRAER,&
 & PABCU, PB , PBINT, PBSUR, PBTOP , PDBSL,&
 & PEMIS, PEMIW,&
 & PGA , PGB , PGASUR,PGBSUR,PGATOP, PGBTOP,&
 & PCNTRB,PFLUC&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOELW , ONLY : NSIL ,NIPD ,NUA
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KUAER
INTEGER(KIND=JPIM),INTENT(IN) :: KTRAER
REAL(KIND=JPRB) ,INTENT(IN) :: PABCU(KLON,NUA,3*KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PB(KLON,NSIL,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PBINT(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PBSUR(KLON,NSIL)
REAL(KIND=JPRB) ,INTENT(IN) :: PBTOP(KLON,NSIL)
REAL(KIND=JPRB) ,INTENT(IN) :: PDBSL(KLON,NSIL,KLEV*2)
REAL(KIND=JPRB) ,INTENT(IN) :: PEMIS(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PEMIW(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PGA(KLON,NIPD,2,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PGB(KLON,NIPD,2,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PGASUR(KLON,NIPD,2)
REAL(KIND=JPRB) ,INTENT(IN) :: PGBSUR(KLON,NIPD,2)
REAL(KIND=JPRB) ,INTENT(IN) :: PGATOP(KLON,NIPD,2)
REAL(KIND=JPRB) ,INTENT(IN) :: PGBTOP(KLON,NIPD,2)
REAL(KIND=JPRB) ,INTENT(OUT) :: PCNTRB(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFLUC(KLON,2,KLEV+1)
END SUBROUTINE LWV
END INTERFACE
