!     ------------------------------------------------------------------
NAMELIST/NAMPHY0/&
  &AERCS1,AERCS3,AERCS5,ALMAV,BEDIFV,ECMNP,ETACUT,AHCLPV &
  &,EDB,EDC,EDD,EVAP,FONT,GALP,GCISMIN,GCCSV,GCVADS,GCVALFA,GCVBETA,GCVMLT &
  &,GCVNU,GCVPSI,GCVPSIE,GCOMOD &
  &,GDDEVA,GDDSDE,GPBLHK0,GPBLHRA,GRCVPP,GWDAMP,GWDBC,GWDCCO,GWDCD &
  &,GWDLT,GWDSE,GWDPROF,GWDVALI,HOBST,HUCOE,HUTIL,HUTIL1,HUTIL2,NPCLO1 &
  &,NPCLO2,QSSC,QSMIN,QSNEBC,QSNEBS,QSSUSC,QSSUSS,QSSUSV,QSUSXC,QSUSXS &
  &,QXRAL,QXRDEL,QXRHX,QXRR,QXRTGH,RPHI0,RPHIR &
  &,RCIN,RCVEVAP,RDTFAC,REVGSL,RTCAPE,RICRLM,RNEGAT,RNLCURV,RRGAMMA,RRSCALE &
  &,SCO,SENSL &
  &,SNNBCO,SPNBCO &
  &,SXNBCO,TCA,TCT,TCW,TENTR,TENTRX,TDDGP,TUDGP,TURB,TVF &
  &,TYM,UHDIFV,USDMLT,USUPRC,USURIC,USURID,USURIDE,USURICE,USURICL,UTILGUST &
  &,VCHRNK,VKARMN,VZ0CM,VZIUSTAR0,XNBMAX,XBLM,XMAXLM,XMINLM,XNBMAX,EDK,XKLM &
  &,ADISE,ADISI,AECLS3,AECLS4,AKN,ALD,ALPHAE,ALPHAT,ECTMIN &
  &,UCWSTAR,UDECT,USHEARM,UPRETMIN,UPRETMAX &
  &,ARSCH,ARSCQ,ARSC1,ARSB2,ACBRPHIM,ALMAVE,RICRET,STTBMIN,AGREKE &
  &,RAUTEFS,RNINTR,RNINTS,RQLCR,RQICRMAX,RQICRMIN,RACCEF,RRIMEF,RAGGEF &
  &,RHCRIT1,RHCRIT2,RETAMIN,RFACNSM,TFVR,TFVS,RAUTEFR,RAUTSBET,GRHCMOD &
  &,RQICRT1,RQICRT2,RQICRSN,RQCRNS,RQICVMAX,RQICVMIN,RQLCV,RHEVAP &
  &,AGRE1,AGRERICR,AJBUMIN,RCOFLM,A0ML_AU,A0ML_AT,A0ML_BU,A0ML_BT &
  &,NUPTKE,GAMTKE,RCOLL,RFALLL &
  &,TDDBU,TDDFR,TUDBU,TUDFR,GCVACHI,GCVALMX,GCVADMW,GCVBEE,GCVEEX,ECMNPI &
  &,GFRIC,GCVSQDN,GCVSQDR,GCVSQDCX,GRRINTE,GRRMINA,GDDBETA,GDDEVF,GDDWPF &
  &,TENTRD &
!     For old ACCVIMP
  &,TENTRVL,TRENTRV &
!     For TKE scheme 
  &,UETEPS,UPRECLP &
  &,ARSC2,ARSCT &
!     For Grenier (2000) top-PBL entrainment scheme 
  &,AGRE2,AGREF &
!     For dry conv. adjustment scheme 
  &,AJ1MEPS,AJ1PEPS,NAJITER &
!     For ACCVIMPGY 
  &,ALFX,TCTC,TVFC,GAMAP1,RKDN,VVN,VVX,FENTRT,HCMIN,FQLIC,FNEBC,FEVAPC &
!
  &,RDPHIC,GWBFAUT,RWBF1,RWBF2,RAUITN,RAUITX,RAUIUSTE &
  &,RSMDNEBX,RSMDTX,NSMTPA,NSMTPB
!     ------------------------------------------------------------------
