MODULE YOMPLDSW

! Platform dependent switches

IMPLICIT NONE

SAVE

LOGICAL :: LOPT_SCALAR
LOGICAL :: LOPT_RS6K
!     ------------------------------------------------------------------
!$OMP THREADPRIVATE(lopt_rs6k,lopt_scalar)
END MODULE YOMPLDSW
