INTERFACE
SUBROUTINE SWTT ( KIDIA, KFDIA, KLON, KNU, KA , PU, PTR)
USE PARKIND1 ,ONLY : JPIM ,JPRB
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KNU
INTEGER(KIND=JPIM),INTENT(IN) :: KA
REAL(KIND=JPRB) ,INTENT(IN) :: PU(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTR(KLON)
END SUBROUTINE SWTT
END INTERFACE
