MODULE iotd_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: imax, jmax, lmax, nid, dim_coord, iotd_ts, iotd_t0, n_names_iotd_def, names_iotd_def, &
            un_nom
  !=======================================================================
  !
  !   Auteur:  F. Hourdin
  !   -------
  !
  !   Objet:
  !   ------
  !   Light interface for netcdf outputs. can be used outside LMDZ
  !
  !=======================================================================

  integer imax, jmax, lmax, nid
  INTEGER dim_coord(4)
  real iotd_ts, iotd_t0
  integer :: n_names_iotd_def
  character*20, dimension(200) :: names_iotd_def
  character*20 :: un_nom

  !$OMP THREADPRIVATE(imax,jmax,lmax,nid,dim_coord,iotd_ts,iotd_t0,n_names_iotd_def,names_iotd_def,&
  !$OMP un_nom)
END MODULE iotd_mod_h
