MODULE tsoilnudge_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: nudge_tsoil, isoil_nudge, Tsoil_nudge, tau_soil_nudge

  LOGICAL nudge_tsoil
  INTEGER isoil_nudge
  REAL Tsoil_nudge, tau_soil_nudge
END MODULE tsoilnudge_mod_h
