INTERFACE
SUBROUTINE SRTM_SPCVRT_MCICA&
 & ( KLEV , KMOL , KSW , KCOLS , PONEMINUS,&
 & PAVEL , PTAVEL , PZ , PTZ , PTBOUND , PALBD , PALBP,&
 & PFRCL , PTAUC , PASYC , POMGC , PTAUA , PASYA , POMGA , PRMU0,&
 & PCOLDRY , PWKL,&
 & KLAYTROP, KLAYSWTCH, KLAYLOW ,&
 & PCO2MULT, PCOLCH4 , PCOLCO2 , PCOLH2O , PCOLMOL , PCOLN2O , PCOLO2 , PCOLO3 ,&
 & PFORFAC , PFORFRAC , KINDFOR , PSELFFAC, PSELFFRAC, KINDSELF ,&
 & PFAC00 , PFAC01 , PFAC10 , PFAC11 ,&
 & KJP , KJT , KJT1 ,&
 & PBBFD, PBBFU, PBBCD, PBBCU ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE PARSRTM , ONLY : JPLAY, JPB1, JPB2, JPGPT
INTEGER(KIND=JPIM),INTENT(IN) :: KSW
INTEGER(KIND=JPIM),INTENT(IN) :: KCOLS
INTEGER(KIND=JPIM) :: KLEV
INTEGER(KIND=JPIM) :: KMOL
REAL(KIND=JPRB) :: PONEMINUS
REAL(KIND=JPRB) :: PAVEL(JPLAY)
REAL(KIND=JPRB) :: PTAVEL(JPLAY)
REAL(KIND=JPRB) :: PZ(0:JPLAY)
REAL(KIND=JPRB) :: PTZ(0:JPLAY)
REAL(KIND=JPRB) :: PTBOUND
REAL(KIND=JPRB) ,INTENT(IN) :: PALBD(KSW)
REAL(KIND=JPRB) ,INTENT(IN) :: PALBP(KSW)
REAL(KIND=JPRB) ,INTENT(IN) :: PFRCL(KCOLS,JPLAY)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAUC(JPLAY,KCOLS)
REAL(KIND=JPRB) ,INTENT(IN) :: PASYC(JPLAY,KCOLS)
REAL(KIND=JPRB) ,INTENT(IN) :: POMGC(JPLAY,KCOLS)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAUA(JPLAY,KSW)
REAL(KIND=JPRB) ,INTENT(IN) :: PASYA(JPLAY,KSW)
REAL(KIND=JPRB) ,INTENT(IN) :: POMGA(JPLAY,KSW)
REAL(KIND=JPRB) :: PRMU0
REAL(KIND=JPRB) :: PCOLDRY(JPLAY)
REAL(KIND=JPRB) :: PWKL(35,JPLAY)
INTEGER(KIND=JPIM) :: KLAYTROP
INTEGER(KIND=JPIM) :: KLAYSWTCH
INTEGER(KIND=JPIM) :: KLAYLOW
REAL(KIND=JPRB) :: PCO2MULT(JPLAY)
REAL(KIND=JPRB) :: PCOLCH4(JPLAY)
REAL(KIND=JPRB) :: PCOLCO2(JPLAY)
REAL(KIND=JPRB) :: PCOLH2O(JPLAY)
REAL(KIND=JPRB) :: PCOLMOL(JPLAY)
REAL(KIND=JPRB) :: PCOLN2O(JPLAY)
REAL(KIND=JPRB) :: PCOLO2(JPLAY)
REAL(KIND=JPRB) :: PCOLO3(JPLAY)
REAL(KIND=JPRB) :: PFORFAC(JPLAY)
REAL(KIND=JPRB) :: PFORFRAC(JPLAY)
INTEGER(KIND=JPIM) :: KINDFOR(JPLAY)
REAL(KIND=JPRB) :: PSELFFAC(JPLAY)
REAL(KIND=JPRB) :: PSELFFRAC(JPLAY)
INTEGER(KIND=JPIM) :: KINDSELF(JPLAY)
REAL(KIND=JPRB) :: PFAC00(JPLAY)
REAL(KIND=JPRB) :: PFAC01(JPLAY)
REAL(KIND=JPRB) :: PFAC10(JPLAY)
REAL(KIND=JPRB) :: PFAC11(JPLAY)
INTEGER(KIND=JPIM) :: KJP(JPLAY)
INTEGER(KIND=JPIM) :: KJT(JPLAY)
INTEGER(KIND=JPIM) :: KJT1(JPLAY)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PBBFD(JPLAY+1)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PBBFU(JPLAY+1)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PBBCD(JPLAY+1)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PBBCU(JPLAY+1)
END SUBROUTINE SRTM_SPCVRT_MCICA
END INTERFACE
