! Replaces yomcst.h

module yomcst_mod_h
  IMPLICIT NONE; PRIVATE

  PUBLIC :: RPI, RCLUM, RHPLA, RKBOL, RNAVO                   &
          , RDAY, REA, REPSM, RSIYEA, RSIDAY, ROMEGA                  &
          , R_ecc, R_peri, R_incl                                      &
          , RA, RG, R1SA                                         &
          , RSIGMA                                                     &
          , R, RMD, RMV, RD, RV, RCPD                    &
          , RMO3, RMCO2, RMC, RMCH4, RMN2O, RMCFC11, RMCFC12        &
          , RCPV, RCVD, RCVV, RKAPPA, RETV, eps_w                    &
          , RCW, RCS                                                 &
          , RLVTT, RLSTT, RLMLT, RTT, RATM                           &
          , RESTT, RALPW, RBETW, RGAMW, RALPS, RBETS, RGAMS            &
          , RALPD, RBETD, RGAMD

  ! A1.0 Fundamental constants
  REAL RPI, RCLUM, RHPLA, RKBOL, RNAVO
  ! A1.1 Astronomical constants
  REAL RDAY, REA, REPSM, RSIYEA, RSIDAY, ROMEGA
  ! A1.1.bis Constantes concernant l'orbite de la Terre:
  REAL R_ecc, R_peri, R_incl
  ! A1.2 Geoide
  REAL RA, RG, R1SA
  ! A1.3 Radiation
  !     REAL RSIGMA,RI0
  REAL RSIGMA
  ! A1.4 Thermodynamic gas phase
  REAL RMO3, RMCO2, RMC, RMCH4, RMN2O, RMCFC11, RMCFC12
  REAL R, RMD, RMV, RD, RV, RCPD, RCPV, RCVD, RCVV
  REAL RKAPPA, RETV, eps_w
  ! A1.5,6 Thermodynamic liquid,solid phases
  REAL RCW, RCS
  ! A1.7 Thermodynamic transition of phase
  REAL RLVTT, RLSTT, RLMLT, RTT, RATM
  ! A1.8 Curve of saturation
  REAL RESTT, RALPW, RBETW, RGAMW, RALPS, RBETS, RGAMS
  REAL RALPD, RBETD, RGAMD

  !$OMP THREADPRIVATE(RPI, RCLUM, RHPLA, RKBOL, RNAVO                   &
  !$OMP      , RDAY, REA, REPSM, RSIYEA, RSIDAY, ROMEGA                  &
  !$OMP      , R_ecc, R_peri, R_incl                                      &
  !$OMP      , RA, RG, R1SA                                         &
  !$OMP      , RSIGMA                                                     &
  !$OMP      , R, RMD, RMV, RD, RV, RCPD                    &
  !$OMP      , RMO3, RMCO2, RMC, RMCH4, RMN2O, RMCFC11, RMCFC12        &
  !$OMP      , RCPV, RCVD, RCVV, RKAPPA, RETV, eps_w                    &
  !$OMP      , RCW, RCS                                                 &
  !$OMP      , RLVTT, RLSTT, RLMLT, RTT, RATM                           &
  !$OMP      , RESTT, RALPW, RBETW, RGAMW, RALPS, RBETS, RGAMS            &
  !$OMP      , RALPD, RBETD, RGAMD)

END MODULE yomcst_mod_h
