! A.I mars 2023

! Declaration des champs inputs a initialiser pour le simulateur d'observable satellite  COSP

    REAL :: zx_rhcosp0(klon,klev), &
            ref_liq_cosp0(klon,klev), ref_ice_cosp0(klon,klev), &
            zx_rh_cosp0(klon,klev), cldfra_cosp0(klon,klev), rnebcon_cosp0(klon,klev), &
            fiwc_cosp0(klon,klev), flwc_cosp0(klon,klev), prfl_cosp0(klon,klev), &
            psfl_cosp0(klon,klev), pmflxr_cosp0(klon,klev), pmflxs_cosp0(klon,klev), &
            mr_ozone_cosp0(klon,klev), cldtau_cosp0(klon,klev), cldemi_cosp0(klon,klev), &
            pctsrf_cosp0(klon),zu10m_cosp0(klon),zv10m_cosp0(klon), &
            zxtsol_cosp0(klon),JrNt_cosp0(klon)

!   COMMON/iniCOSP/                                                 &
!            zx_rhcosp0(klon,klev),ref_liq_cosp0(klon,klev), ref_ice_cosp0(klon,klev), &
!	    zx_rh_cosp0(klon,klev), cldfra_cosp0(klon,klev), rnebcon_cosp0(klon,klev), &
!            fiwc_cosp0(klon,klev), flwc_cosp0(klon,klev), prfl_cosp0(klon,klev), &
!            psfl_cosp0(klon,klev), pmflxr_cosp0(klon,klev), pmflxs_cosp0(klon,klev), &
!            mr_ozone_cosp0(klon,klev), cldtau_cosp0(klon,klev), cldemi_cosp0(klon,klev), &
!            pctsrf_cosp0(klon),zu10m_cosp0(klon),zv10m_cosp0(klon), &
!            zxtsol_cosp0(klon),JrNt_cosp0(klon)
!	save /iniCOSP/
!!$OMP THREADPRIVATE(/iniCOSP/)
	
