!!!! Abderrahmane Idelkadi aout 2013 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Module pour definir (au 1er appel) et ecrire les variables dans les fichiers de sortie cosp
!
! R.Guzman jan 2019 (mise a jour pour COSPv2)
! On change le nom du module a "lmdz_cosp_output_write_mod" et celui de la routine a "lmdz_cosp_output_write"
! pour qu on sache qu il s agit d un module specifique a l implementation de COSP dans LMDZ
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   MODULE lmdz_cosp_output_write_mod

   USE lmdz_cosp_output_mod
   USE mod_cosp_config, only : R_UNDEF, CLOUDSAT_DBZE_BINS, SR_BINS, PARASOL_NREFL, & 
                              isccp_histPresCenters,tau_binCenters, LIDAR_NTEMP, & 
                              LIDAR_PHASE_TEMP,misr_histHgtCenters,numMISRHgtBins, & 
                              numMISRTauBins,numMODISReffIceBins,reffICE_binCenters, &
                              numMODISReffLiqBins, reffLIQ_binCenters

   IMPLICIT NONE

   INTEGER, SAVE  :: itau_iocosp
!$OMP THREADPRIVATE(itau_iocosp)
   INTEGER, save        :: Nlevout, Ncolout
!$OMP THREADPRIVATE(Nlevout, Ncolout)

!  INTERFACE histwrite_cosp
!    MODULE PROCEDURE histwrite2d_cosp,histwrite3d_cosp
!  END INTERFACE

   CONTAINS

  SUBROUTINE lmdz_cosp_output_write(Nlevlmdz, Npoints, Ncolumns, itap, dtime, freq_COSP, &
                                    missing_val, cfg, Nlvgrid, cospOUT)


    USE ioipsl
    USE time_phylmdz_mod, ONLY: itau_phy, start_time, day_step_phy
    USE print_control_mod, ONLY: lunout,prt_level
    USE lmdz_cosp_read_outputkeys, only: cosp_config
!COSPv2
    use cosp_kinds, only: wp
    use mod_cosp,   only: cosp_outputs

    use wxios_mod, only: wxios_closedef
    USE lmdz_xios, only: xios_update_calendar, xios_field_is_active

! prb Sorties Modis
    use, intrinsic :: ieee_arithmetic

  IMPLICIT NONE  
!!! Variables d'entree
  integer, intent(in)   :: itap, Nlevlmdz, Ncolumns, Npoints, Nlvgrid
  real, intent(in)      :: freq_COSP, dtime,  missing_val
  type(cosp_config), intent(in)   :: cfg     ! Control outputs
  type(cosp_outputs), intent(inout)    :: &
       cospOUT           ! COSP simulator outputs


!!! Variables locales
  integer               :: icl,k,ip
  logical               :: ok_sync
  integer               :: itau_wcosp, iff
  real, dimension(Npoints,PARASOL_NREFL) :: parasolcrefl, Ncref

! Variables locals intermidiaires pour inverser les axes des champs 4D
! Compatibilite avec sorties CMIP
  real, dimension(Npoints,Nlvgrid,SR_BINS) :: tmp_fi4da_cfadL, tmp_fi4da_cfadLgr, tmp_fi4da_cfadLatlid
  real, dimension(Npoints,Nlvgrid,CLOUDSAT_DBZE_BINS) :: tmp_fi4da_cfadR
  real, dimension(Npoints,numMISRHgtBins,numMISRTauBins) :: tmp_fi4da_misr
  !real, dimension(Npoints,numMODISReffIceBins,numMODISTauBins) :: tmp_fi4da_modis

  !missing_val=missing_cosp

  Nlevout = Nlvgrid
  Ncolout = Ncolumns

! A refaire
       itau_wcosp = itau_phy + itap + start_time * day_step_phy
        if (prt_level >= 10) then
             WRITE(lunout,*)'itau_wcosp, itap, start_time, day_step_phy =', & 
                             itau_wcosp, itap, start_time, day_step_phy
        endif

! On le donne a  cosp_output_write_mod pour que les histwrite y aient acces:
       CALL set_itau_iocosp(itau_wcosp)
        if (prt_level >= 10) then
              WRITE(lunout,*)'itau_iocosp =',itau_iocosp
        endif

    ok_sync = .TRUE.
    
!!!! Sorties Calipso
 if (cfg%Lcalipso) then

!   print*,'Appel histwrite2d_cosp'
   if (cfg%Lcllcalipso) then 
      where(cospOUT%calipso_cldlayer(:,1) == R_UNDEF) cospOUT%calipso_cldlayer(:,1) = missing_val
      CALL histwrite2d_cosp(o_cllcalipso,cospOUT%calipso_cldlayer(:,1))
   endif
   if (cfg%Lclhcalipso) then 
      where(cospOUT%calipso_cldlayer(:,3) == R_UNDEF) cospOUT%calipso_cldlayer(:,3) = missing_val
      CALL histwrite2d_cosp(o_clhcalipso,cospOUT%calipso_cldlayer(:,3))
   endif
   if (cfg%Lclmcalipso) then
      where(cospOUT%calipso_cldlayer(:,2) == R_UNDEF) cospOUT%calipso_cldlayer(:,2) = missing_val
      CALL histwrite2d_cosp(o_clmcalipso,cospOUT%calipso_cldlayer(:,2)) 
   endif
   if (cfg%Lcltcalipso) then
      where(cospOUT%calipso_cldlayer(:,4) == R_UNDEF) cospOUT%calipso_cldlayer(:,4) = missing_val
      CALL histwrite2d_cosp(o_cltcalipso,cospOUT%calipso_cldlayer(:,4))
   endif
   if (cfg%Lclcalipso) then 
      where(cospOUT%calipso_lidarcld == R_UNDEF)    cospOUT%calipso_lidarcld = missing_val
      CALL histwrite3d_cosp(o_clcalipso,cospOUT%calipso_lidarcld,nvert)
   endif
   if (cfg%Lclcalipsotmp) then
      where(cospOUT%calipso_lidarcldtmp(:,:,1) == R_UNDEF) cospOUT%calipso_lidarcldtmp(:,:,1) = missing_val
      CALL histwrite3d_cosp(o_clcalipsotmp,cospOUT%calipso_lidarcldtmp(:,:,1),nverttemp)
   endif

   if (cfg%Lcllcalipsoice) then
      where(cospOUT%calipso_cldlayerphase(:,1,1) == R_UNDEF) cospOUT%calipso_cldlayerphase(:,1,1) = missing_val
      CALL histwrite2d_cosp(o_cllcalipsoice,cospOUT%calipso_cldlayerphase(:,1,1))
   endif
   if (cfg%Lclhcalipsoice) then
      where(cospOUT%calipso_cldlayerphase(:,3,1) == R_UNDEF) cospOUT%calipso_cldlayerphase(:,3,1) = missing_val 
      CALL histwrite2d_cosp(o_clhcalipsoice,cospOUT%calipso_cldlayerphase(:,3,1))
   endif
   if (cfg%Lclmcalipsoice) then
      where(cospOUT%calipso_cldlayerphase(:,2,1) == R_UNDEF) cospOUT%calipso_cldlayerphase(:,2,1) = missing_val
      CALL histwrite2d_cosp(o_clmcalipsoice,cospOUT%calipso_cldlayerphase(:,2,1))
   endif
   if (cfg%Lcltcalipsoice) then
      where(cospOUT%calipso_cldlayerphase(:,4,1) == R_UNDEF) cospOUT%calipso_cldlayerphase(:,4,1) = missing_val
      CALL histwrite2d_cosp(o_cltcalipsoice,cospOUT%calipso_cldlayerphase(:,4,1))
   endif
   if (cfg%Lclcalipsoice) then 
      where(cospOUT%calipso_lidarcldphase(:,:,1) == R_UNDEF) cospOUT%calipso_lidarcldphase(:,:,1) = missing_val
      CALL histwrite3d_cosp(o_clcalipsoice,cospOUT%calipso_lidarcldphase(:,:,1),nvert)
   endif
   if (cfg%Lclcalipsotmpice) then
      where(cospOUT%calipso_lidarcldtmp(:,:,2) == R_UNDEF) cospOUT%calipso_lidarcldtmp(:,:,2) = missing_val
      CALL histwrite3d_cosp(o_clcalipsotmpice,cospOUT%calipso_lidarcldtmp(:,:,2),nverttemp)
   endif

   if (cfg%Lcllcalipsoliq) then
      where(cospOUT%calipso_cldlayerphase(:,1,2) == R_UNDEF) cospOUT%calipso_cldlayerphase(:,1,2) = missing_val
      CALL histwrite2d_cosp(o_cllcalipsoliq,cospOUT%calipso_cldlayerphase(:,1,2))
   endif
   if (cfg%Lclhcalipsoliq) then 
      where(cospOUT%calipso_cldlayerphase(:,3,2) == R_UNDEF) cospOUT%calipso_cldlayerphase(:,3,2) = missing_val
      CALL histwrite2d_cosp(o_clhcalipsoliq,cospOUT%calipso_cldlayerphase(:,3,2))
   endif
   if (cfg%Lclmcalipsoliq) then 
      where(cospOUT%calipso_cldlayerphase(:,2,2) == R_UNDEF) cospOUT%calipso_cldlayerphase(:,2,2) = missing_val
      CALL histwrite2d_cosp(o_clmcalipsoliq,cospOUT%calipso_cldlayerphase(:,2,2))
   endif
   if (cfg%Lcltcalipsoliq) then
      where(cospOUT%calipso_cldlayerphase(:,4,2) == R_UNDEF) cospOUT%calipso_cldlayerphase(:,4,2) = missing_val
      CALL histwrite2d_cosp(o_cltcalipsoliq,cospOUT%calipso_cldlayerphase(:,4,2))
   endif
   if (cfg%Lclcalipsoliq) then
      where(cospOUT%calipso_lidarcldphase(:,:,2) == R_UNDEF) cospOUT%calipso_lidarcldphase(:,:,2) = missing_val
      CALL histwrite3d_cosp(o_clcalipsoliq,cospOUT%calipso_lidarcldphase(:,:,2),nvert)
   endif
   if (cfg%Lclcalipsotmpliq) then
      where(cospOUT%calipso_lidarcldtmp(:,:,3) == R_UNDEF) cospOUT%calipso_lidarcldtmp(:,:,3) = missing_val
      CALL histwrite3d_cosp(o_clcalipsotmpliq,cospOUT%calipso_lidarcldtmp(:,:,3),nverttemp)
   endif
   if (cfg%Lcllcalipsoun) then
      where(cospOUT%calipso_cldlayerphase(:,1,3) == R_UNDEF) cospOUT%calipso_cldlayerphase(:,1,3) = missing_val
      CALL histwrite2d_cosp(o_cllcalipsoun,cospOUT%calipso_cldlayerphase(:,1,3))
   endif 
   if (cfg%Lclhcalipsoun) then
      where(cospOUT%calipso_cldlayerphase(:,3,3) == R_UNDEF) cospOUT%calipso_cldlayerphase(:,3,3) = missing_val
      CALL histwrite2d_cosp(o_clhcalipsoun,cospOUT%calipso_cldlayerphase(:,3,3))
   endif
   if (cfg%Lclmcalipsoun) then 
      where(cospOUT%calipso_cldlayerphase(:,2,3) == R_UNDEF) cospOUT%calipso_cldlayerphase(:,2,3) = missing_val
      CALL histwrite2d_cosp(o_clmcalipsoun,cospOUT%calipso_cldlayerphase(:,2,3))
   endif
   if (cfg%Lcltcalipsoun) then
      where(cospOUT%calipso_cldlayerphase(:,4,3) == R_UNDEF) cospOUT%calipso_cldlayerphase(:,4,3) = missing_val
      CALL histwrite2d_cosp(o_cltcalipsoun,cospOUT%calipso_cldlayerphase(:,4,3))
   endif
   if (cfg%Lclcalipsoun) then 
      where(cospOUT%calipso_lidarcldphase(:,:,3) == R_UNDEF) cospOUT%calipso_lidarcldphase(:,:,3) = missing_val
      CALL histwrite3d_cosp(o_clcalipsoun,cospOUT%calipso_lidarcldphase(:,:,3),nvert)
   endif
   if (cfg%Lclcalipsotmpun) then
      where(cospOUT%calipso_lidarcldtmp(:,:,4) == R_UNDEF) cospOUT%calipso_lidarcldtmp(:,:,4) = missing_val
      CALL histwrite3d_cosp(o_clcalipsotmpun,cospOUT%calipso_lidarcldtmp(:,:,4),nverttemp)
   endif

   if (cfg%Lclopaquecalipso) then 
      where(cospOUT%calipso_cldtype(:,1) == R_UNDEF) cospOUT%calipso_cldtype(:,1) = missing_val
      CALL histwrite2d_cosp(o_clopaquecalipso,cospOUT%calipso_cldtype(:,1))
   endif
   if (cfg%Lclthincalipso) then
      where(cospOUT%calipso_cldtype(:,2) == R_UNDEF) cospOUT%calipso_cldtype(:,2) = missing_val
      CALL histwrite2d_cosp(o_clthincalipso,cospOUT%calipso_cldtype(:,2))   
   endif
   if (cfg%Lclzopaquecalipso) then
      where(cospOUT%calipso_cldtype(:,3) == R_UNDEF) cospOUT%calipso_cldtype(:,3) = missing_val      
      CALL histwrite2d_cosp(o_clzopaquecalipso,cospOUT%calipso_cldtype(:,3))
   endif
   if (cfg%Lclcalipsoopaque) then
      where(cospOUT%calipso_lidarcldtype(:,:,1) == R_UNDEF) cospOUT%calipso_lidarcldtype(:,:,1) = missing_val      
      CALL histwrite3d_cosp(o_clcalipsoopaque,cospOUT%calipso_lidarcldtype(:,:,1),nvert)
   endif
   if (cfg%Lclcalipsothin) then
      where(cospOUT%calipso_lidarcldtype(:,:,2) == R_UNDEF) cospOUT%calipso_lidarcldtype(:,:,2) = missing_val
      CALL histwrite3d_cosp(o_clcalipsothin,cospOUT%calipso_lidarcldtype(:,:,2),nvert)  
   endif
   if (cfg%Lclcalipsozopaque) then
      where(cospOUT%calipso_lidarcldtype(:,:,3) == R_UNDEF) cospOUT%calipso_lidarcldtype(:,:,3) = missing_val
      CALL histwrite3d_cosp(o_clcalipsozopaque,cospOUT%calipso_lidarcldtype(:,:,3),nvert)
   endif
   if (cfg%Lclcalipsoopacity) then
      where(cospOUT%calipso_lidarcldtype(:,:,4) == R_UNDEF) cospOUT%calipso_lidarcldtype(:,:,4) = missing_val
      CALL histwrite3d_cosp(o_clcalipsoopacity,cospOUT%calipso_lidarcldtype(:,:,4),nvert)
   endif

   if (cfg%Lclopaquetemp) then 
      where(cospOUT%calipso_cldtypetemp(:,1) == R_UNDEF) cospOUT%calipso_cldtypetemp(:,1) = missing_val
      CALL histwrite2d_cosp(o_clopaquetemp,cospOUT%calipso_cldtypetemp(:,1))
   endif
   if (cfg%Lclthintemp) then 
      where(cospOUT%calipso_cldtypetemp(:,2) == R_UNDEF) cospOUT%calipso_cldtypetemp(:,2) = missing_val
      CALL histwrite2d_cosp(o_clthintemp,cospOUT%calipso_cldtypetemp(:,2))
   endif 
   if (cfg%Lclzopaquetemp) then
      where(cospOUT%calipso_cldtypetemp(:,3) == R_UNDEF) cospOUT%calipso_cldtypetemp(:,3) = missing_val
      CALL histwrite2d_cosp(o_clzopaquetemp,cospOUT%calipso_cldtypetemp(:,3))
   endif
   if (cfg%Lclopaquemeanz) then
      where(cospOUT%calipso_cldtypemeanz(:,1) == R_UNDEF) cospOUT%calipso_cldtypemeanz(:,1) = missing_val
      CALL histwrite2d_cosp(o_clopaquemeanz,cospOUT%calipso_cldtypemeanz(:,1))
   endif
   if (cfg%Lclthinmeanz) then
      where(cospOUT%calipso_cldtypemeanz(:,2) == R_UNDEF) cospOUT%calipso_cldtypemeanz(:,2) = missing_val
      CALL histwrite2d_cosp(o_clthinmeanz,cospOUT%calipso_cldtypemeanz(:,2))
   endif
   if (cfg%Lclthinemis) then 
      where(cospOUT%calipso_cldthinemis == R_UNDEF) cospOUT%calipso_cldthinemis = missing_val
      CALL histwrite2d_cosp(o_clthinemis,cospOUT%calipso_cldthinemis)
   endif
   if (cfg%Lclopaquemeanzse) then
      where(cospOUT%calipso_cldtypemeanzse(:,1) == R_UNDEF) cospOUT%calipso_cldtypemeanzse(:,1) = missing_val
      CALL histwrite2d_cosp(o_clopaquemeanzse,cospOUT%calipso_cldtypemeanzse(:,1))
   endif
   if (cfg%Lclthinmeanzse) then
      where(cospOUT%calipso_cldtypemeanzse(:,2) == R_UNDEF) cospOUT%calipso_cldtypemeanzse(:,2) = missing_val
      CALL histwrite2d_cosp(o_clthinmeanzse,cospOUT%calipso_cldtypemeanzse(:,2))
   endif
   if (cfg%Lclzopaquecalipsose) then 
      where(cospOUT%calipso_cldtypemeanzse(:,3) == R_UNDEF) cospOUT%calipso_cldtypemeanzse(:,3) = missing_val
      CALL histwrite2d_cosp(o_clzopaquecalipsose,cospOUT%calipso_cldtypemeanzse(:,3))
   endif


   if (cfg%LcfadLidarsr532) then 
     where(cospOUT%calipso_cfad_sr == R_UNDEF) cospOUT%calipso_cfad_sr = missing_val
    
     do icl=1,SR_BINS
      do k=1,Nlvgrid
       do ip=1,Npoints
        tmp_fi4da_cfadL(ip,k,icl)=cospOUT%calipso_cfad_sr(ip,icl,k)
       enddo
      enddo
     enddo
!   if (cfg%LcfadLidarsr532) CALL histwrite4d_cosp(o_cfad_lidarsr532,stlidar%cfad_sr)
    CALL histwrite4d_cosp(o_cfadLidarsr532,tmp_fi4da_cfadL)   !!! "_" enleve
   endif

   if (cfg%Latb532) then  
     where(cospOUT%calipso_beta_tot == R_UNDEF) cospOUT%calipso_beta_tot = missing_val
     CALL histwrite4d_cosp(o_atb532,cospOUT%calipso_beta_tot)
   endif
   if (cfg%LlidarBetaMol532) then 
     where(cospOUT%calipso_beta_mol == R_UNDEF) cospOUT%calipso_beta_mol = missing_val
     CALL histwrite3d_cosp(o_lidarBetaMol532,cospOUT%calipso_beta_mol,nvertmcosp) 
   endif

 endif !Calipso


!!!! Sorties Ground Lidar
 if (cfg%LgrLidar532) then
! AI juin 2020 Voir a quoi correspond ce champs
!   where(cospOUT%grLidar532_srbval == R_UNDEF) cospOUT%grLidar532_srbval = missing_val

   if (cfg%LcllgrLidar532) then
      where(cospOUT%grLidar532_cldlayer(:,1) == R_UNDEF) cospOUT%grLidar532_cldlayer(:,1) = missing_val
      CALL histwrite2d_cosp(o_cllgrLidar532,cospOUT%grLidar532_cldlayer(:,1))
   endif
   if (cfg%LclmgrLidar532) then
      where(cospOUT%grLidar532_cldlayer(:,2) == R_UNDEF) cospOUT%grLidar532_cldlayer(:,2) = missing_val
      CALL histwrite2d_cosp(o_clmgrLidar532,cospOUT%grLidar532_cldlayer(:,2))
   endif
   if (cfg%LclhgrLidar532) then
      where(cospOUT%grLidar532_cldlayer(:,3) == R_UNDEF) cospOUT%grLidar532_cldlayer(:,3) = missing_val
      CALL histwrite2d_cosp(o_clhgrLidar532,cospOUT%grLidar532_cldlayer(:,3))
   endif
   if (cfg%LcltgrLidar532) then
      where(cospOUT%grLidar532_cldlayer(:,4) == R_UNDEF) cospOUT%grLidar532_cldlayer(:,4) = missing_val
      CALL histwrite2d_cosp(o_cltgrLidar532,cospOUT%grLidar532_cldlayer(:,4))
   endif

   if (cfg%LclgrLidar532) then
       where(cospOUT%grLidar532_lidarcld == R_UNDEF) cospOUT%grLidar532_lidarcld = missing_val
      CALL histwrite3d_cosp(o_clgrLidar532,cospOUT%grLidar532_lidarcld,nvert)
   endif
   if (cfg%LlidarBetaMol532gr) then
      where(cospOUT%grLidar532_beta_mol == R_UNDEF) cospOUT%grLidar532_beta_mol = missing_val
      CALL histwrite3d_cosp(o_lidarBetaMol532gr,cospOUT%grLidar532_beta_mol,nvertmcosp) 
   endif
   if (cfg%LcfadLidarsr532gr) then
     where(cospOUT%grLidar532_cfad_sr == R_UNDEF) cospOUT%grLidar532_cfad_sr = missing_val
     do icl=1,SR_BINS
      do k=1,Nlvgrid
       do ip=1,Npoints
         tmp_fi4da_cfadLgr(ip,k,icl)=cospOUT%grLidar532_cfad_sr(ip,icl,k)
       enddo
      enddo
     enddo
     CALL histwrite4d_cosp(o_cfadLidarsr532gr,tmp_fi4da_cfadLgr)
   endif

   if (cfg%Latb532gr) then
      where(cospOUT%grLidar532_beta_tot == R_UNDEF) cospOUT%grLidar532_beta_tot = missing_val
      CALL histwrite4d_cosp(o_atb532gr,cospOUT%grLidar532_beta_tot)
   endif
endif ! Ground Lidar 532 nm


!!!! Sorties Atlid
 if (cfg%Latlid) then
! AI juin 2020 Voir a quoi correspond ce champs
!   where(cospOUT%atlid_srbval == R_UNDEF) cospOUT%atlid_srbval = missing_val

   if (cfg%Lcllatlid) then 
      where(cospOUT%atlid_cldlayer(:,1) == R_UNDEF) cospOUT%atlid_cldlayer(:,1) = missing_val
      CALL histwrite2d_cosp(o_cllatlid,cospOUT%atlid_cldlayer(:,1))
   endif
   if (cfg%Lclmatlid) then
      where(cospOUT%atlid_cldlayer(:,2) == R_UNDEF) cospOUT%atlid_cldlayer(:,2) = missing_val
      CALL histwrite2d_cosp(o_clmatlid,cospOUT%atlid_cldlayer(:,2))
   endif
   if (cfg%Lclhatlid) then
      where(cospOUT%atlid_cldlayer(:,3) == R_UNDEF) cospOUT%atlid_cldlayer(:,3) = missing_val
      CALL histwrite2d_cosp(o_clhatlid,cospOUT%atlid_cldlayer(:,3))
   endif
   if (cfg%Lcltatlid) then
      where(cospOUT%atlid_cldlayer(:,4) == R_UNDEF) cospOUT%atlid_cldlayer(:,4) = missing_val
      CALL histwrite2d_cosp(o_cltatlid,cospOUT%atlid_cldlayer(:,4))
   endif
   if (cfg%Lclatlid) then
      where(cospOUT%atlid_lidarcld == R_UNDEF) cospOUT%atlid_lidarcld = missing_val
      CALL histwrite3d_cosp(o_clatlid,cospOUT%atlid_lidarcld,nvert)
   endif
   if (cfg%LlidarBetaMol355) then
      where(cospOUT%atlid_beta_mol == R_UNDEF) cospOUT%atlid_beta_mol = missing_val
      CALL histwrite3d_cosp(o_lidarBetaMol355,cospOUT%atlid_beta_mol,nvertmcosp) 
   endif
   if (cfg%LcfadLidarsr355) then
     where(cospOUT%atlid_cfad_sr == R_UNDEF) cospOUT%atlid_cfad_sr = missing_val
     do icl=1,SR_BINS
      do k=1,Nlvgrid
       do ip=1,Npoints
          tmp_fi4da_cfadLatlid(ip,k,icl)=cospOUT%atlid_cfad_sr(ip,icl,k)
       enddo
      enddo
     enddo
     CALL histwrite4d_cosp(o_cfadlidarsr355,tmp_fi4da_cfadLatlid)
   endif

   if (cfg%Latb355) then
      where(cospOUT%atlid_beta_tot == R_UNDEF) cospOUT%atlid_beta_tot = missing_val 
      CALL histwrite4d_cosp(o_atb355,cospOUT%atlid_beta_tot)
   endif
endif ! Atlid


 if (cfg%Lparasol) then
   if (cfg%LparasolRefl) then
     where(cospOUT%parasolGrid_refl == R_UNDEF) cospOUT%parasolGrid_refl = missing_val
     where(cospOUT%parasolPix_refl == R_UNDEF) cospOUT%parasolPix_refl = missing_val
     CALL histwrite3d_cosp(o_parasolGrid_refl,cospOUT%parasolGrid_refl,nvertp)
     CALL histwrite4d_cosp(o_parasolPix_refl,cospOUT%parasolPix_refl)
   endif ! LparasolRefl
 endif ! Parasol

!  if (cfg%LparasolRefl) then 
!    do k=1,PARASOL_NREFL
!     do ip=1, Npoints
!      if (stlidar%cldlayer(ip,4).gt.1.and.stlidar%parasolrefl(ip,k).ne.missing_val) then
!        parasolcrefl(ip,k)=(stlidar%parasolrefl(ip,k)-0.03*(1.-stlidar%cldlayer(ip,4)/100.))/ &
!                             (stlidar%cldlayer(ip,4)/100.)
!         Ncref(ip,k) = 1.
!      else
!         parasolcrefl(ip,k)=missing_val
!         Ncref(ip,k) = 0.
!      endif
!     enddo
!    enddo
!    CALL histwrite3d_cosp(o_Ncrefl,Ncref,nvertp)
!    CALL histwrite3d_cosp(o_parasol_crefl,parasolcrefl,nvertp)
!  endif


!!! Sorties CloudSat
 if (cfg%Lcloudsat) then
! AI juin 2020 voir a quoi correspond ce champs
!   where(cospOUT%cloudsat_pia == R_UNDEF) cospOUT%cloudsat_pia = missing_val

   if (cfg%Lptradarflag0) then
     where(cospOUT%cloudsat_precip_cover(:,1) == R_UNDEF) cospOUT%cloudsat_precip_cover(:,1) = missing_val
     CALL histwrite2d_cosp(o_ptradarflag0,cospOUT%cloudsat_precip_cover(:,1))
   endif
   if (cfg%Lptradarflag1) then
     where(cospOUT%cloudsat_precip_cover(:,2) == R_UNDEF) cospOUT%cloudsat_precip_cover(:,2) = missing_val
     CALL histwrite2d_cosp(o_ptradarflag1,cospOUT%cloudsat_precip_cover(:,2))
   endif
   if (cfg%Lptradarflag2) then
      where(cospOUT%cloudsat_precip_cover(:,3) == R_UNDEF) cospOUT%cloudsat_precip_cover(:,3) = missing_val
      CALL histwrite2d_cosp(o_ptradarflag2,cospOUT%cloudsat_precip_cover(:,3))
   endif
   if (cfg%Lptradarflag3) then
      where(cospOUT%cloudsat_precip_cover(:,4) == R_UNDEF) cospOUT%cloudsat_precip_cover(:,4) = missing_val
      CALL histwrite2d_cosp(o_ptradarflag3,cospOUT%cloudsat_precip_cover(:,4))
   endif
   if (cfg%Lptradarflag4) then
      where(cospOUT%cloudsat_precip_cover(:,5) == R_UNDEF) cospOUT%cloudsat_precip_cover(:,5) = missing_val
      CALL histwrite2d_cosp(o_ptradarflag3,cospOUT%cloudsat_precip_cover(:,5))
   endif
   if (cfg%Lptradarflag5) then
      where(cospOUT%cloudsat_precip_cover(:,6) == R_UNDEF) cospOUT%cloudsat_precip_cover(:,6) = missing_val
      CALL histwrite2d_cosp(o_ptradarflag3,cospOUT%cloudsat_precip_cover(:,6))
   endif
   if (cfg%Lptradarflag6) then
      where(cospOUT%cloudsat_precip_cover(:,7) == R_UNDEF) cospOUT%cloudsat_precip_cover(:,7) = missing_val
      CALL histwrite2d_cosp(o_ptradarflag3,cospOUT%cloudsat_precip_cover(:,7))
   endif
   if (cfg%Lptradarflag7) then
      where(cospOUT%cloudsat_precip_cover(:,8) == R_UNDEF) cospOUT%cloudsat_precip_cover(:,8) = missing_val
      CALL histwrite2d_cosp(o_ptradarflag3,cospOUT%cloudsat_precip_cover(:,8))
   endif
   if (cfg%Lptradarflag8) then
      where(cospOUT%cloudsat_precip_cover(:,9) == R_UNDEF) cospOUT%cloudsat_precip_cover(:,9) = missing_val
      CALL histwrite2d_cosp(o_ptradarflag3,cospOUT%cloudsat_precip_cover(:,9))
   endif
   if (cfg%Lptradarflag9) then
      where(cospOUT%cloudsat_precip_cover(:,10) == R_UNDEF) cospOUT%cloudsat_precip_cover(:,10) = missing_val
      CALL histwrite2d_cosp(o_ptradarflag3,cospOUT%cloudsat_precip_cover(:,10))
   endif

   if (cfg%Ldbze94) then
     where(cospOUT%cloudsat_Ze_tot == R_UNDEF) cospOUT%cloudsat_Ze_tot = missing_val 
     CALL histwrite4d_cosp(o_dbze94,cospOUT%cloudsat_Ze_tot)
   endif
   if (cfg%LcfadDbze94)  then
     where(cospOUT%cloudsat_cfad_ze == R_UNDEF) cospOUT%cloudsat_cfad_ze = missing_val  
     do icl=1,CLOUDSAT_DBZE_BINS
      do k=1,Nlvgrid
       do ip=1,Npoints
         tmp_fi4da_cfadR(ip,k,icl)=cospOUT%cloudsat_cfad_ze(ip,icl,k)
       enddo
      enddo
     enddo
     CALL histwrite4d_cosp(o_cfadDbze94,tmp_fi4da_cfadR)
  endif
 endif
! endif pour CloudSat


!!! Sorties combinees Cloudsat et Calipso
 if (cfg%Lcalipso .and. cfg%Lcloudsat) then

   if (cfg%Lclcalipso2) then
     where(cospOUT%lidar_only_freq_cloud == R_UNDEF) &
                           cospOUT%lidar_only_freq_cloud = missing_val
     CALL histwrite3d_cosp(o_clcalipso2,cospOUT%lidar_only_freq_cloud,nvert)
   endif
   if (cfg%Lcloudsat_tcc) then
     where(cospOUT%cloudsat_tcc == R_UNDEF) &
                           cospOUT%cloudsat_tcc = missing_val
     CALL histwrite2d_cosp(o_cloudsat_tcc,cospOUT%cloudsat_tcc)
   endif
   if (cfg%Lcloudsat_tcc2) then
     where(cospOUT%cloudsat_tcc2 == R_UNDEF) &
                           cospOUT%cloudsat_tcc2 = missing_val
     CALL histwrite2d_cosp(o_cloudsat_tcc2,cospOUT%cloudsat_tcc2)
   endif
   if (cfg%Lcltlidarradar) then
     where(cospOUT%radar_lidar_tcc == R_UNDEF) &
                           cospOUT%radar_lidar_tcc = missing_val     
     CALL histwrite2d_cosp(o_cltlidarradar,cospOUT%radar_lidar_tcc)
   endif
 endif


!!! Sorties Isccp
 if (cfg%Lisccp) then
!   CALL histwrite2d_cosp(o_sunlit,gbx%sunlit)
  if (cfg%Lclisccp) then
    where(cospOUT%isccp_fq == R_UNDEF) cospOUT%isccp_fq = missing_val
    CALL histwrite4d_cosp(o_clisccp,cospOUT%isccp_fq)
  endif        
  if (cfg%Lboxtauisccp) then
    where(cospOUT%isccp_boxtau == R_UNDEF) cospOUT%isccp_boxtau = missing_val
    CALL histwrite3d_cosp(o_boxtauisccp,cospOUT%isccp_boxtau,nvertcol)
  endif
  if (cfg%Lboxptopisccp) then
    where(cospOUT%isccp_boxptop == R_UNDEF) cospOUT%isccp_boxptop = missing_val
    CALL histwrite3d_cosp(o_boxptopisccp,cospOUT%isccp_boxptop,nvertcol) 
  endif
  if (cfg%Lcltisccp) then
    where(cospOUT%isccp_totalcldarea == R_UNDEF) cospOUT%isccp_totalcldarea = missing_val  
    CALL histwrite2d_cosp(o_cltisccp,cospOUT%isccp_totalcldarea) 
  endif
  if (cfg%Lpctisccp) then
    where(cospOUT%isccp_meanptop == R_UNDEF) cospOUT%isccp_meanptop = missing_val      
    CALL histwrite2d_cosp(o_pctisccp,cospOUT%isccp_meanptop) 
  endif
  if (cfg%Ltauisccp) then
    where(cospOUT%isccp_meantaucld == R_UNDEF) cospOUT%isccp_meantaucld = missing_val   
    CALL histwrite2d_cosp(o_tauisccp,cospOUT%isccp_meantaucld) 
  endif
  if (cfg%Lalbisccp) then
    where(cospOUT%isccp_meanalbedocld == R_UNDEF) cospOUT%isccp_meanalbedocld = missing_val   
    CALL histwrite2d_cosp(o_albisccp,cospOUT%isccp_meanalbedocld) 
  endif
  if (cfg%Lmeantbisccp) then
    where(cospOUT%isccp_meantb == R_UNDEF) cospOUT%isccp_meantb = missing_val      
    CALL histwrite2d_cosp(o_meantbisccp,cospOUT%isccp_meantb) 
  endif
  if (cfg%Lmeantbclrisccp) then
    where(cospOUT%isccp_meantbclr == R_UNDEF) cospOUT%isccp_meantbclr = missing_val      
    CALL histwrite2d_cosp(o_meantbclrisccp,cospOUT%isccp_meantbclr)
  endif
 endif ! Isccp


!!! MISR simulator
 if (cfg%Lmisr) then

   if (cfg%LclMISR) then
! Ces 3 diagnostics sont controles par la clef logique "LclMISR"
   where(cospOUT%misr_fq == R_UNDEF) cospOUT%misr_fq = missing_val
!   where(cospOUT%misr_dist_model_layertops == R_UNDEF) cospOUT%misr_dist_model_layertops = missing_val
   where(cospOUT%misr_meanztop == R_UNDEF) cospOUT%misr_meanztop = missing_val
   where(cospOUT%misr_cldarea == R_UNDEF) cospOUT%misr_cldarea = missing_val

   do icl=1,numMISRHgtBins
      do k=1,numMISRTauBins
       do ip=1,Npoints    
      tmp_fi4da_misr(ip,icl,k)=cospOUT%misr_fq(ip,k,icl)
       enddo
      enddo
   enddo
!   if (cfg%LclMISR) CALL histwrite4d_cosp(o_clMISR,misr%fq_MISR)
!   if (cfg%LclMISR) CALL histwrite4d_cosp(o_clMISR,tmp_fi4da_misr)
   CALL histwrite4d_cosp(o_misr_fq,tmp_fi4da_misr)

   CALL histwrite2d_cosp(o_misr_meanztop,cospOUT%misr_meanztop) 
   CALL histwrite2d_cosp(o_misr_cldarea,cospOUT%misr_cldarea*100.) 
  endif ! LclMISR

 endif ! Misr

!!! Modis simulator
 if (cfg%Lmodis) then
  if (cfg%Lcllmodis) then
    where(cospOUT%modis_Cloud_Fraction_Low_Mean == R_UNDEF) &
        cospOUT%modis_Cloud_Fraction_Low_Mean = missing_val
    CALL histwrite2d_cosp(o_cllmodis,cospOUT%modis_Cloud_Fraction_Low_Mean)
  endif
  if (cfg%Lclhmodis) then
    where(cospOUT%modis_Cloud_Fraction_High_Mean == R_UNDEF) &
        cospOUT%modis_Cloud_Fraction_High_Mean = missing_val
    CALL histwrite2d_cosp(o_clhmodis,cospOUT%modis_Cloud_Fraction_High_Mean)
  endif
  if (cfg%Lclmmodis) then
    where(cospOUT%modis_Cloud_Fraction_Mid_Mean == R_UNDEF) &
        cospOUT%modis_Cloud_Fraction_Mid_Mean = missing_val
    CALL histwrite2d_cosp(o_clmmodis,cospOUT%modis_Cloud_Fraction_Mid_Mean)
  endif
  if (cfg%Lcltmodis) then
    where(cospOUT%modis_Cloud_Fraction_Total_Mean == R_UNDEF) &
        cospOUT%modis_Cloud_Fraction_Total_Mean = missing_val
    CALL histwrite2d_cosp(o_cltmodis,cospOUT%modis_Cloud_Fraction_Total_Mean)
  endif
  if (cfg%Lclwmodis) then
    where(cospOUT%modis_Cloud_Fraction_Water_Mean == R_UNDEF) &
        cospOUT%modis_Cloud_Fraction_Water_Mean = missing_val
    CALL histwrite2d_cosp(o_clwmodis,cospOUT%modis_Cloud_Fraction_Water_Mean)
  endif
  if (cfg%Lclimodis) then
    where(cospOUT%modis_Cloud_Fraction_Ice_Mean == R_UNDEF) &
        cospOUT%modis_Cloud_Fraction_Ice_Mean = missing_val
    CALL histwrite2d_cosp(o_climodis,cospOUT%modis_Cloud_Fraction_Ice_Mean)
  endif
  if (cfg%Ltautmodis) then
    where(cospOUT%modis_Optical_Thickness_Total_Mean == R_UNDEF) &
        cospOUT%modis_Optical_Thickness_Total_Mean = missing_val
    CALL histwrite2d_cosp(o_tautmodis,cospOUT%modis_Optical_Thickness_Total_Mean)
  endif
  if (cfg%Ltauwmodis) then
    where(cospOUT%modis_Optical_Thickness_Water_Mean == R_UNDEF) &
        cospOUT%modis_Optical_Thickness_Water_Mean = missing_val
    CALL histwrite2d_cosp(o_tauwmodis,cospOUT%modis_Optical_Thickness_Water_Mean)
  endif
  if (cfg%Ltauimodis) then
    where(cospOUT%modis_Optical_Thickness_Ice_Mean == R_UNDEF) &
        cospOUT%modis_Optical_Thickness_Ice_Mean = missing_val
    CALL histwrite2d_cosp(o_tauimodis,cospOUT%modis_Optical_Thickness_Ice_Mean)
  endif
  if (cfg%Lreffclwmodis) then
    where(cospOUT%modis_Cloud_Particle_Size_Water_Mean == R_UNDEF) &
        cospOUT%modis_Cloud_Particle_Size_Water_Mean = missing_val
    CALL histwrite2d_cosp(o_reffclwmodis,cospOUT%modis_Cloud_Particle_Size_Water_Mean)
  endif 
  if (cfg%Lreffclimodis) then
    where(cospOUT%modis_Cloud_Particle_Size_Ice_Mean == R_UNDEF) &
        cospOUT%modis_Cloud_Particle_Size_Ice_Mean = missing_val
    CALL histwrite2d_cosp(o_reffclimodis,cospOUT%modis_Cloud_Particle_Size_Ice_Mean)
  endif
  if (cfg%Ltauilogmodis) then
    where(cospOUT%modis_Optical_Thickness_Ice_LogMean == R_UNDEF) &
        cospOUT%modis_Optical_Thickness_Ice_LogMean = missing_val
    CALL histwrite2d_cosp(o_tauilogmodis,cospOUT%modis_Optical_Thickness_Ice_LogMean)
  endif
  if (cfg%Ltauwlogmodis) then
    where(cospOUT%modis_Optical_Thickness_Water_LogMean == R_UNDEF) &
        cospOUT%modis_Optical_Thickness_Water_LogMean = missing_val
    CALL histwrite2d_cosp(o_lwpmodis,cospOUT%modis_Optical_Thickness_Water_LogMean)
  endif
  if (cfg%Ltautlogmodis) then 
    where(cospOUT%modis_Optical_Thickness_Total_LogMean == R_UNDEF) &
        cospOUT%modis_Optical_Thickness_Total_LogMean = missing_val
    CALL histwrite2d_cosp(o_iwpmodis,cospOUT%modis_Optical_Thickness_Total_LogMean)
  endif
  if (cfg%Lpctmodis) then
    where(cospOUT%modis_Cloud_Top_Pressure_Total_Mean == R_UNDEF) &
        cospOUT%modis_Cloud_Top_Pressure_Total_Mean = missing_val
    CALL histwrite2d_cosp(o_pctmodis,cospOUT%modis_Cloud_Top_Pressure_Total_Mean)
  endif  
  if (cfg%Llwpmodis) then
    where(cospOUT%modis_Liquid_Water_Path_Mean == R_UNDEF) &
        cospOUT%modis_Liquid_Water_Path_Mean = missing_val
    CALL histwrite2d_cosp(o_lwpmodis,cospOUT%modis_Liquid_Water_Path_Mean)
  endif
  if (cfg%Liwpmodis) then
    where(cospOUT%modis_Ice_Water_Path_Mean == R_UNDEF) &
        cospOUT%modis_Ice_Water_Path_Mean = missing_val
    CALL histwrite2d_cosp(o_iwpmodis,cospOUT%modis_Ice_Water_Path_Mean)
  endif
  if (cfg%Lclmodis) then
! Ces 3 diagnostics sont controles par la clef logique "Lclmodis"
!AI attention plantage pour ces 2 champs Modis 
! Valeurs Nan, champs non initialises 
! Initialisation a R_UNDEF dans cosp.F90
! a mettre a jour Modis
! valeur NaN exclues a revoir
    !print *, 'shape = ', shape(cospOUT%modis_Optical_Thickness_vs_ReffICE)
    !print *, 'min = ', minval(cospOUT%modis_Optical_Thickness_vs_ReffICE)
    !print *, 'max = ', maxval(cospOUT%modis_Optical_Thickness_vs_ReffICE)
  ! 1) Traiter d'abord les NaN / Inf
   !where (.not. ieee_is_finite(cospOUT%modis_Optical_Thickness_vs_ReffICE))
    !  cospOUT%modis_Optical_Thickness_vs_ReffICE = missing_val
   !end where
! 2) Ensuite seulement les valeurs R_UNDEF
   !where (cospOUT%modis_Optical_Thickness_vs_ReffICE == R_UNDEF)
    !  cospOUT%modis_Optical_Thickness_vs_ReffICE = missing_val
   !end where

     ! 1) Traiter d'abord les NaN / Inf
   !where (.not. ieee_is_finite(cospOUT%modis_Optical_Thickness_vs_ReffLIQ))
    !  cospOUT%modis_Optical_Thickness_vs_ReffLIQ = missing_val
   !end where
! 2) Ensuite seulement les valeurs R_UNDEF
   !where (cospOUT%modis_Optical_Thickness_vs_ReffLIQ == R_UNDEF)
    !  cospOUT%modis_Optical_Thickness_vs_ReffLIQ = missing_val
   !end where

!AI partie originale ou plantait
   !if (any(.not. ieee_is_finite(cospOUT%modis_Optical_Thickness_vs_ReffICE))) then
    !  print *, 'ERROR: NaN or Inf detected in modis_Optical_Thickness_vs_ReffICE'
     ! stop
   !endif

    where(cospOUT%modis_Optical_Thickness_vs_Cloud_Top_Pressure == R_UNDEF) &
          cospOUT%modis_Optical_Thickness_vs_Cloud_Top_Pressure = missing_val
    where(cospOUT%modis_Optical_Thickness_vs_ReffICE == R_UNDEF) &
          cospOUT%modis_Optical_Thickness_vs_ReffICE = missing_val
    where(cospOUT%modis_Optical_thickness_vs_ReffLIQ == R_UNDEF) &
          cospOUT%modis_Optical_thickness_vs_ReffLIQ = missing_val
    CALL histwrite4d_cosp(o_modis_ot_vs_ctp,cospOUT%modis_Optical_Thickness_vs_Cloud_Top_Pressure)
    CALL histwrite4d_cosp(o_modis_ot_vs_reffice,cospOUT%modis_Optical_Thickness_vs_ReffICE)
    CALL histwrite4d_cosp(o_modis_ot_vs_reffliq,cospOUT%modis_Optical_thickness_vs_ReffLIQ)

  endif ! Lclmodis

 endif !modis


 IF(.NOT.cosp_varsdefined) THEN
!$OMP MASTER
! Fermeture dans phys_output_write
!#ifdef 1
            !On finalise l'initialisation:
            !CALL wxios_closedef()
!#endif

!$OMP END MASTER
!$OMP BARRIER
            cosp_varsdefined = .TRUE.
 END IF

    IF(cosp_varsdefined) THEN
! On synchronise les fichiers pour IOIPSL
    ENDIF  !cosp_varsdefined

    END SUBROUTINE lmdz_cosp_output_write


! ug Routine pour definir itau_iocosp depuis cosp_output_write_mod:
  SUBROUTINE set_itau_iocosp(ito)
      USE clesphys_mod_h
    IMPLICIT NONE
      INTEGER, INTENT(IN) :: ito
      itau_iocosp = ito
  END SUBROUTINE

  SUBROUTINE histdef2d_cosp (iff,var)

    USE ioipsl
    USE dimphy
    use iophy
    USE mod_phys_lmdz_para
    USE mod_grid_phy_lmdz, ONLY: nbp_lon
    USE print_control_mod, ONLY: lunout,prt_level
    use wxios_mod
    USE clesphys_mod_h, ONLY: ok_all_xml

    IMPLICIT NONE


    INTEGER                          :: iff
    TYPE(ctrl_outcosp)               :: var

    REAL zstophym
    CHARACTER(LEN=20) :: typeecrit

    ! ug On récupère le type écrit de la structure:
    !       Assez moche, �|  refaire si meilleure méthode...
    IF (INDEX(var%cosp_typeecrit(iff), "once") > 0) THEN
       typeecrit = 'once'
    ELSE IF(INDEX(var%cosp_typeecrit(iff), "t_min") > 0) THEN
       typeecrit = 't_min(X)'
    ELSE IF(INDEX(var%cosp_typeecrit(iff), "t_max") > 0) THEN
       typeecrit = 't_max(X)'
    ELSE IF(INDEX(var%cosp_typeecrit(iff), "inst") > 0) THEN
       typeecrit = 'inst(X)'
    ELSE
       typeecrit = cosp_outfiletypes(iff)
    ENDIF

    IF (typeecrit=='inst(X)'.OR.typeecrit=='once') THEN
       zstophym=zoutm_cosp(iff)
    ELSE
       zstophym=zdtimemoy_cosp
    ENDIF

     IF (.not. ok_all_xml) then
       IF ( var%cles(iff) ) THEN
         if (prt_level >= 10) then
              WRITE(lunout,*)'Appel wxios_add_field_to_file var%name =',var%name 
         endif
        CALL wxios_add_field_to_file(var%name, 2, cosp_nidfiles(iff), cosp_outfilenames(iff), &
                                     var%description, var%unit, 1, typeecrit)
       ENDIF
     ENDIF


  END SUBROUTINE histdef2d_cosp


 SUBROUTINE histdef3d_cosp (iff,var,nvertsave,ncols)
    USE clesphys_mod_h
    USE ioipsl
    USE dimphy
    use iophy
    USE mod_phys_lmdz_para
    USE mod_grid_phy_lmdz, ONLY: nbp_lon
    USE print_control_mod, ONLY: lunout,prt_level

  use wxios_mod


    IMPLICIT NONE


    INTEGER                        :: iff, klevs
    INTEGER, INTENT(IN), OPTIONAL  :: ncols ! ug RUSTINE POUR LES variables 4D
    INTEGER, INTENT(IN)           :: nvertsave
    TYPE(ctrl_outcosp)             :: var

    REAL zstophym
    CHARACTER(LEN=20) :: typeecrit, nomi
    CHARACTER(LEN=20) :: nom
    character(len=2) :: str2
    CHARACTER(len=20) :: nam_axvert

! Axe vertical
      IF (nvertsave.eq.nvertp(iff)) THEN
          klevs=PARASOL_NREFL
          nam_axvert="sza"
      ELSE IF (nvertsave.eq.nvertisccp(iff)) THEN
          klevs=7
          nam_axvert="pressure2"
      ELSE IF (nvertsave.eq.nvertcol(iff)) THEN
          klevs=Ncolout
          nam_axvert="column"
      ELSE IF (nvertsave.eq.nverttemp(iff)) THEN
          klevs=LIDAR_NTEMP
          nam_axvert="temp"
      ELSE IF (nvertsave.eq.nvertmisr(iff)) THEN
          klevs=numMISRHgtBins
          nam_axvert="cth16"
      ELSE IF (nvertsave.eq.nvertReffIce(iff)) THEN
          klevs= numMODISReffIceBins
          nam_axvert="ReffIce"
      ELSE IF (nvertsave.eq.nvertReffLiq(iff)) THEN
          klevs= numMODISReffLiqBins
          nam_axvert="ReffLiq"
      ELSE
           klevs=Nlevout
           nam_axvert="presnivs"
      ENDIF

! ug RUSTINE POUR LES Champs 4D
      IF (PRESENT(ncols)) THEN
               write(str2,'(i2.2)')ncols
               nomi=var%name
               nom="c"//str2//"_"//nomi
      ELSE
               nom=var%name
      END IF

    ! ug On récupère le type écrit de la structure:
    !       Assez moche, �|  refaire si meilleure méthode...
    IF (INDEX(var%cosp_typeecrit(iff), "once") > 0) THEN
       typeecrit = 'once'
    ELSE IF(INDEX(var%cosp_typeecrit(iff), "t_min") > 0) THEN
       typeecrit = 't_min(X)'
    ELSE IF(INDEX(var%cosp_typeecrit(iff), "t_max") > 0) THEN
       typeecrit = 't_max(X)'
    ELSE IF(INDEX(var%cosp_typeecrit(iff), "inst") > 0) THEN
       typeecrit = 'inst(X)'
    ELSE
       typeecrit = cosp_outfiletypes(iff)
    ENDIF

    IF (typeecrit=='inst(X)'.OR.typeecrit=='once') THEN
       zstophym=zoutm_cosp(iff)
    ELSE
       zstophym=zdtimemoy_cosp
    ENDIF

      IF (.not. ok_all_xml) then
        IF ( var%cles(iff) ) THEN
          if (prt_level >= 10) then
              WRITE(lunout,*)'Appel wxios_add_field_to_file 3d nom variable nam_axvert = ',nom, nam_axvert 
          endif
          CALL wxios_add_field_to_file(nom, 3, cosp_nidfiles(iff), cosp_outfilenames(iff), &
                                       var%description, var%unit, 1, typeecrit, nam_axvert)
        ENDIF
      ENDIF


  END SUBROUTINE histdef3d_cosp


 SUBROUTINE histwrite2d_cosp(var,field)
  USE clesphys_mod_h
  USE dimphy
  USE mod_phys_lmdz_para
  USE ioipsl
  use iophy
  USE mod_grid_phy_lmdz, ONLY: nbp_lon
  USE print_control_mod, ONLY: lunout,prt_level

  USE lmdz_xios, only: xios_send_field

  IMPLICIT NONE

    TYPE(ctrl_outcosp), INTENT(IN) :: var
    REAL, DIMENSION(:), INTENT(IN) :: field

    INTEGER :: iff

    REAL,DIMENSION(klon_mpi) :: buffer_omp
    INTEGER, allocatable, DIMENSION(:) :: index2d
    REAL :: Field2d(nbp_lon,jj_nb)
    CHARACTER(LEN=20) ::  nomi, nom
    character(len=2) :: str2
    LOGICAL, SAVE  :: firstx
!$OMP THREADPRIVATE(firstx)

    IF (prt_level >= 9) WRITE(lunout,*)'Begin histrwrite2d ',var%name

  ! On regarde si on est dans la phase de définition ou d'écriture:
  IF(.NOT.cosp_varsdefined) THEN
!$OMP MASTER
      print*,'var, cosp_varsdefined dans cosp_varsdefined ',var%name, cosp_varsdefined
      !Si phase de définition.... on définit
      CALL conf_cospoutputs(var%name,var%cles)
      DO iff=1, 3
         IF (cosp_outfilekeys(iff)) THEN
            CALL histdef2d_cosp(iff, var)
         ENDIF
      ENDDO
!$OMP END MASTER
  ELSE
    !Et sinon on.... écrit
    IF (SIZE(field)/=klon) &
  CALL abort_physic('iophy::histwrite2d_cosp','Field first DIMENSION not equal to klon',1) 

    CALL Gather_omp(field,buffer_omp)
!$OMP MASTER
    CALL grid1Dto2D_mpi(buffer_omp,Field2d)

! La boucle sur les fichiers:
      firstx=.true.
      DO iff=1, 3
           IF (var%cles(iff) .AND. cosp_outfilekeys(iff)) THEN
                ALLOCATE(index2d(nbp_lon*jj_nb))
                deallocate(index2d)
              IF (.not. ok_all_xml) then
                 if (firstx) then
                  if (prt_level >= 10) then
                    WRITE(lunout,*)'xios_send_field variable ',var%name
                  endif
                  CALL xios_send_field(var%name, Field2d)
                   firstx=.false.
                 endif
              ENDIF
           ENDIF
      ENDDO 

      IF (ok_all_xml) THEN
        if (prt_level >= 1) then
              WRITE(lunout,*)'xios_send_field variable ',var%name
        endif
       CALL xios_send_field(var%name, Field2d)
      ENDIF

!$OMP END MASTER   
  ENDIF ! vars_defined
  IF (prt_level >= 9) WRITE(lunout,*)'End histrwrite2d_cosp ',var%name
  END SUBROUTINE histwrite2d_cosp


! ug NOUVELLE VERSION DES WRITE AVEC LA BOUCLE DO RENTREE
! AI sept 2013
  SUBROUTINE histwrite3d_cosp(var, field, nverts, ncols)
  USE clesphys_mod_h
  USE dimphy
  USE mod_phys_lmdz_para
  USE ioipsl
  use iophy
  USE mod_grid_phy_lmdz, ONLY: nbp_lon
  USE print_control_mod, ONLY: lunout,prt_level

  USE lmdz_xios, only: xios_send_field


  IMPLICIT NONE

    TYPE(ctrl_outcosp), INTENT(IN)    :: var
    REAL, DIMENSION(:,:), INTENT(IN)  :: field ! --> field(klon,:)
    INTEGER, INTENT(IN), OPTIONAL     :: ncols ! ug RUSTINE POUR LES Champs 4D.....
    INTEGER, DIMENSION(3), INTENT(IN) :: nverts

    INTEGER :: iff, k

    REAL,DIMENSION(klon_mpi,SIZE(field,2)) :: buffer_omp
    REAL :: Field3d(nbp_lon,jj_nb,SIZE(field,2))
    INTEGER :: ip, n, nlev
    INTEGER, ALLOCATABLE, DIMENSION(:) :: index3d
    CHARACTER(LEN=20) ::  nomi, nom
    character(len=2) :: str2
    LOGICAL, SAVE  :: firstx
!$OMP THREADPRIVATE(firstx)

  IF (prt_level >= 9) write(lunout,*)'Begin histrwrite3d ',var%name

! ug RUSTINE POUR LES STD LEVS.....
      IF (PRESENT(ncols)) THEN
              write(str2,'(i2.2)')ncols
              nomi=var%name
              nom="c"//str2//"_"//nomi
      ELSE
               nom=var%name
      END IF
  ! On regarde si on est dans la phase de définition ou d'écriture:
  IF(.NOT.cosp_varsdefined) THEN
      !Si phase de définition.... on définit
!$OMP MASTER
      CALL conf_cospoutputs(var%name,var%cles)
      DO iff=1, 3
        IF (cosp_outfilekeys(iff)) THEN
          CALL histdef3d_cosp(iff, var, nverts(iff), ncols)
        ENDIF
      ENDDO
!$OMP END MASTER
  ELSE
    !Et sinon on.... écrit
    IF (SIZE(field,1)/=klon) &
   CALL abort_physic('iophy::histwrite3d','Field first DIMENSION not equal to klon',1)                                  
    nlev=SIZE(field,2)


    CALL Gather_omp(field,buffer_omp)
!$OMP MASTER
    CALL grid1Dto2D_mpi(buffer_omp,field3d)

! BOUCLE SUR LES FICHIERS
     firstx=.true.
     DO iff=1, 3
        IF (var%cles(iff) .AND. cosp_outfilekeys(iff)) THEN
           ALLOCATE(index3d(nbp_lon*jj_nb*nlev))

          IF (.not. ok_all_xml) then
           IF (firstx) THEN
               CALL xios_send_field(nom, Field3d(:,:,1:nlev))
               IF (prt_level >= 9) WRITE(lunout,*)'xios_send_field ',var%name
               firstx=.FALSE.
           ENDIF
          ENDIF
         deallocate(index3d)
        ENDIF
      ENDDO
    IF (ok_all_xml) THEN
     CALL xios_send_field(nom, Field3d(:,:,1:nlev))
     IF (prt_level >= 1) WRITE(lunout,*)'xios_send_field ',var%name
    ENDIF

!$OMP END MASTER   
  ENDIF ! vars_defined
  IF (prt_level >= 9) write(lunout,*)'End histrwrite3d_cosp ',nom
  END SUBROUTINE histwrite3d_cosp


! ug NOUVELLE VERSION DES WRITE AVEC LA BOUCLE DO RENTREE
! AI sept 2013
  SUBROUTINE histwrite4d_cosp(var, field)
  USE clesphys_mod_h
  USE dimphy
  USE mod_phys_lmdz_para
  USE ioipsl
  use iophy
  USE mod_grid_phy_lmdz, ONLY: nbp_lon
  USE print_control_mod, ONLY: lunout,prt_level

  USE lmdz_xios, only: xios_send_field


  IMPLICIT NONE

    TYPE(ctrl_outcosp), INTENT(IN)    :: var
    REAL, DIMENSION(:,:,:), INTENT(IN)  :: field ! --> field(klon,:)

    INTEGER :: iff, k

    REAL,DIMENSION(klon_mpi,SIZE(field,2),SIZE(field,3)) :: buffer_omp
    REAL :: field4d(nbp_lon,jj_nb,SIZE(field,2),SIZE(field,3))
    INTEGER :: ip, n, nlev, nlev2
    INTEGER, ALLOCATABLE, DIMENSION(:) :: index4d
    CHARACTER(LEN=20) ::  nomi, nom

  IF (prt_level >= 9) write(lunout,*)'Begin histrwrite4d ',var%name

  IF(cosp_varsdefined) THEN
    !Et sinon on.... écrit
    IF (SIZE(field,1)/=klon) &
   CALL abort_physic('iophy::histwrite3d','Field first DIMENSION not equal to klon',1)            

    nlev=SIZE(field,2)
    nlev2=SIZE(field,3)
    CALL Gather_omp(field,buffer_omp)
!$OMP MASTER
    CALL grid1Dto2D_mpi(buffer_omp,field4d)

!    IF (ok_all_xml) THEN
     CALL xios_send_field(var%name, Field4d(:,:,1:nlev,1:nlev2))
     IF (prt_level >= 1) WRITE(lunout,*)'xios_send_field ',var%name
!    ENDIF

!$OMP END MASTER   
  ENDIF ! vars_defined
  IF (prt_level >= 9) write(lunout,*)'End histrwrite4d_cosp ',nom
  END SUBROUTINE histwrite4d_cosp

  SUBROUTINE conf_cospoutputs(nam_var,cles_var)
!!! Lecture des noms et cles de sortie des variables dans config.def
    !   en utilisant les routines getin de IOIPSL  
    use ioipsl
    USE print_control_mod, ONLY: lunout,prt_level

    IMPLICIT NONE

   CHARACTER(LEN=20)               :: nam_var, nnam_var
   LOGICAL, DIMENSION(3)           :: cles_var

! Lecture dans config.def ou output.def de cles_var et name_var
    CALL getin('cles_'//nam_var,cles_var)
    CALL getin('name_'//nam_var,nam_var)
    IF(prt_level>10) WRITE(lunout,*)'nam_var cles_var ',nam_var,cles_var(:)

  END SUBROUTINE conf_cospoutputs

 END MODULE lmdz_cosp_output_write_mod
