INTERFACE
SUBROUTINE GPPREF(KPROMA,KSTART,KPROF,KFLEV,PVAH,PVBH,PALPH,PRESH,PRESF)
USE PARKIND1 ,ONLY : JPIM ,JPRB
INTEGER(KIND=JPIM),INTENT(IN) :: KPROMA
INTEGER(KIND=JPIM),INTENT(IN) :: KFLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KSTART
INTEGER(KIND=JPIM),INTENT(IN) :: KPROF
REAL(KIND=JPRB) :: PVAH(0:KFLEV)
REAL(KIND=JPRB) :: PVBH(0:KFLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PALPH(KPROMA,KFLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PRESH(KPROMA,0:KFLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRESF(KPROMA,KFLEV)
END SUBROUTINE GPPREF
END INTERFACE
