INTERFACE
SUBROUTINE RDCSET(CDSL,KSLWIDEN,KSLWIDES,KSLWIDEW,KSLWIDEE,&
 & KSLRPTSUR,KSLSPTSUR,&
 & KDGLG,KDLON,KDGSAG,KDGENG,KDGUXL,KDGUXG,KDGSAL,KDGENL,&
 & KDSUR1,KDLSUR,KDGSUR,KGPTOT,KGPTOT_CAP,&
 & KPTRFLOFF,KFRSTLOFF,KYFRSTACTLAT,KYLSTACTLAT,&
 & KSTA,KONL,KLOENG,KPTRFRSTLAT,KFRSTLAT,KLSTLAT,&
 & PMU,PSQM2,&
 & KSLSTA,KSLONL,KSLOFF,KSLEXT,KSLCORE,KASLB1,&
 & KSLPROCS,KSLMPBUFSZ,KSLRPT,KSLSPT,&
 & KSLSENDPOS,KSLRECVPOS,KSENDPTR,KRECVPTR,KSLCOMM,KMAP,KMAPLEN) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOMCT0 , ONLY : NPROC ,NPRINTLEV,LOUTPUT ,LMPDIAG ,LALLOPR ,&
 & LELAM ,N_REGIONS_NS ,N_REGIONS_EW 
INTEGER(KIND=JPIM),INTENT(IN) :: KSLWIDEN
INTEGER(KIND=JPIM),INTENT(IN) :: KSLWIDES
INTEGER(KIND=JPIM),INTENT(IN) :: KSLWIDEW
INTEGER(KIND=JPIM),INTENT(IN) :: KSLWIDEE
INTEGER(KIND=JPIM),INTENT(IN) :: KSLRPTSUR
INTEGER(KIND=JPIM),INTENT(IN) :: KSLSPTSUR
INTEGER(KIND=JPIM),INTENT(IN) :: KDGLG
INTEGER(KIND=JPIM),INTENT(IN) :: KDLON
INTEGER(KIND=JPIM),INTENT(IN) :: KDGSAG
INTEGER(KIND=JPIM),INTENT(IN) :: KDGENG
INTEGER(KIND=JPIM),INTENT(IN) :: KDGSAL
INTEGER(KIND=JPIM),INTENT(IN) :: KDGENL
INTEGER(KIND=JPIM),INTENT(IN) :: KGPTOT
CHARACTER(LEN=2) ,INTENT(IN) :: CDSL
INTEGER(KIND=JPIM),INTENT(IN) :: KDGUXL
INTEGER(KIND=JPIM),INTENT(IN) :: KDGUXG
INTEGER(KIND=JPIM),INTENT(IN) :: KDSUR1
INTEGER(KIND=JPIM),INTENT(IN) :: KDLSUR
INTEGER(KIND=JPIM),INTENT(IN) :: KDGSUR
INTEGER(KIND=JPIM),INTENT(IN) :: KGPTOT_CAP
INTEGER(KIND=JPIM),INTENT(IN) :: KPTRFLOFF
INTEGER(KIND=JPIM),INTENT(IN) :: KFRSTLOFF
INTEGER(KIND=JPIM),INTENT(IN) :: KYFRSTACTLAT
INTEGER(KIND=JPIM),INTENT(IN) :: KYLSTACTLAT
INTEGER(KIND=JPIM),INTENT(IN) :: KSTA(KDGSAG:KDGENG+N_REGIONS_NS-1,N_REGIONS_EW)
INTEGER(KIND=JPIM),INTENT(IN) :: KONL(KDGSAG:KDGENG+N_REGIONS_NS-1,N_REGIONS_EW)
INTEGER(KIND=JPIM),INTENT(IN) :: KLOENG(KDGSAG:KDGENG)
INTEGER(KIND=JPIM),INTENT(IN) :: KPTRFRSTLAT(N_REGIONS_NS)
INTEGER(KIND=JPIM),INTENT(IN) :: KFRSTLAT(N_REGIONS_NS)
INTEGER(KIND=JPIM),INTENT(IN) :: KLSTLAT(N_REGIONS_NS)
REAL(KIND=JPRB) ,INTENT(IN) :: PMU(KDGSAG:KDGENG)
REAL(KIND=JPRB) ,INTENT(IN) :: PSQM2(KDGSAG:KDGENG)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSLSTA(KDGSAL-KSLWIDEN:KDGENL+KSLWIDES)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSLONL(KDGSAL-KSLWIDEN:KDGENL+KSLWIDES)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSLOFF(KDGSAL-KSLWIDEN:KDGENL+KSLWIDES)
INTEGER(KIND=JPIM),INTENT(OUT) :: KSLEXT(1-KDLON:KDLON+KDLON,1-KSLWIDEN:KDGENL+KSLWIDES)
INTEGER(KIND=JPIM),INTENT(OUT) :: KSLCORE(KGPTOT)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KASLB1
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSLPROCS
INTEGER(KIND=JPIM),INTENT(OUT) :: KSLMPBUFSZ
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSLRPT
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSLSPT
INTEGER(KIND=JPIM),INTENT(OUT) :: KSLSENDPOS(KSLSPTSUR)
INTEGER(KIND=JPIM),INTENT(OUT) :: KSLRECVPOS(KSLRPTSUR)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSENDPTR(NPROC+1)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KRECVPTR(NPROC+1)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSLCOMM(NPROC)
INTEGER(KIND=JPIM),INTENT(OUT) :: KMAP(4,KDGLG)
INTEGER(KIND=JPIM),INTENT(OUT) :: KMAPLEN
END SUBROUTINE RDCSET
END INTERFACE
