!     ------------------------------------------------------------------
NAMELIST/NAMPHY3/&
  &BSFSA,BSFSI,BSFSN,BSFTA,BSFTI,BSFTN,EARRT,EOASA,EOASI,EOASN &
  &,EOATA,EOATI,EOATN,EODSA,EODSI,EODSN,EODTA,EODTI,EODTN,EORAY &
  &,GCA,GCB,GCC,GCD4,GCE4,GIREC1,GIREC2,GIREC3,GIREC4 &
  &,QCO2,QLIMI,QLIP0,RII0,USAA,USAI,USAN,USBA,USBI,USBN,VDP,VNP &
  &,FCM_DEL_A,FCM_DEL_D,FCM_MU_A,FCM_MU_D,FCM_N_I,FCM_N_L &
  &,FCM_P_AI,FCM_P_AL,FCM_P_DI,FCM_P_DL,FCM_P_GI,FCM_P_GL &
  &,FCM_Q_AI,FCM_Q_AL,FCM_Q_DI,FCM_Q_DL,FCM_Q_GI,FCM_Q_GL &
  &,REXP_NEB
!     ------------------------------------------------------------------

