MODULE YOERRTM

USE PARKIND1  ,ONLY : JPIM
USE PARRRTM   ,ONLY : JPGMAX

IMPLICIT NONE

PUBLIC

SAVE

!     ------------------------------------------------------------------
!     Parameters relevant to AER's RRTM-LW radiation scheme: Part 2

!     20110613 JJMorcrette

!     Modified to allow possibilities of different g-point numbers.  
!     ------------------------------------------------------------------

!INTEGER(KIND=JPIM) :: JPGPT
!INTEGER(KIND=JPIM) :: JPGLW

!INTEGER(KIND=JPIM) :: NG1
!INTEGER(KIND=JPIM) :: NG2
!INTEGER(KIND=JPIM) :: NG3
!INTEGER(KIND=JPIM) :: NG4
!INTEGER(KIND=JPIM) :: NG5
!INTEGER(KIND=JPIM) :: NG6
!INTEGER(KIND=JPIM) :: NG7
!INTEGER(KIND=JPIM) :: NG8
!INTEGER(KIND=JPIM) :: NG9
!INTEGER(KIND=JPIM) :: NG10
!INTEGER(KIND=JPIM) :: NG11
!INTEGER(KIND=JPIM) :: NG12
!INTEGER(KIND=JPIM) :: NG13
!INTEGER(KIND=JPIM) :: NG14
!INTEGER(KIND=JPIM) :: NG15
!INTEGER(KIND=JPIM) :: NG16

!INTEGER(KIND=JPIM) :: NGS1
!INTEGER(KIND=JPIM) :: NGS2
!INTEGER(KIND=JPIM) :: NGS3
!INTEGER(KIND=JPIM) :: NGS4
!INTEGER(KIND=JPIM) :: NGS5
!INTEGER(KIND=JPIM) :: NGS6
!INTEGER(KIND=JPIM) :: NGS7
!INTEGER(KIND=JPIM) :: NGS8
!INTEGER(KIND=JPIM) :: NGS9
!INTEGER(KIND=JPIM) :: NGS10
!INTEGER(KIND=JPIM) :: NGS11
!INTEGER(KIND=JPIM) :: NGS12
!INTEGER(KIND=JPIM) :: NGS13
!INTEGER(KIND=JPIM) :: NGS14
!INTEGER(KIND=JPIM) :: NGS15
!INTEGER(KIND=JPIM) :: NGS16

INTEGER(KIND=JPIM), PARAMETER :: JPGPT  = 140
INTEGER(KIND=JPIM), PARAMETER :: JPGLW  = 140

!-- NGnn : number of g-points in each longwave spectral band
INTEGER(KIND=JPIM), PARAMETER :: NG1  = 10
INTEGER(KIND=JPIM), PARAMETER :: NG2  = 12
INTEGER(KIND=JPIM), PARAMETER :: NG3  = 16
INTEGER(KIND=JPIM), PARAMETER :: NG4  = 14
INTEGER(KIND=JPIM), PARAMETER :: NG5  = 16
INTEGER(KIND=JPIM), PARAMETER :: NG6  = 8
INTEGER(KIND=JPIM), PARAMETER :: NG7  = 12
INTEGER(KIND=JPIM), PARAMETER :: NG8  = 8
INTEGER(KIND=JPIM), PARAMETER :: NG9  = 12
INTEGER(KIND=JPIM), PARAMETER :: NG10 = 6
INTEGER(KIND=JPIM), PARAMETER :: NG11 = 8
INTEGER(KIND=JPIM), PARAMETER :: NG12 = 8
INTEGER(KIND=JPIM), PARAMETER :: NG13 = 4
INTEGER(KIND=JPIM), PARAMETER :: NG14 = 2
INTEGER(KIND=JPIM), PARAMETER :: NG15 = 2
INTEGER(KIND=JPIM), PARAMETER :: NG16 = 2
!-- NGSnn: accumulated number of g-points at the beginning of spectral band nn+1
INTEGER(KIND=JPIM), PARAMETER :: NGS1  = 10
INTEGER(KIND=JPIM), PARAMETER :: NGS2  = 22
INTEGER(KIND=JPIM), PARAMETER :: NGS3  = 38
INTEGER(KIND=JPIM), PARAMETER :: NGS4  = 52
INTEGER(KIND=JPIM), PARAMETER :: NGS5  = 68
INTEGER(KIND=JPIM), PARAMETER :: NGS6  = 76
INTEGER(KIND=JPIM), PARAMETER :: NGS7  = 88
INTEGER(KIND=JPIM), PARAMETER :: NGS8  = 96
INTEGER(KIND=JPIM), PARAMETER :: NGS9  = 108
INTEGER(KIND=JPIM), PARAMETER :: NGS10 = 114
INTEGER(KIND=JPIM), PARAMETER :: NGS11 = 122
INTEGER(KIND=JPIM), PARAMETER :: NGS12 = 130
INTEGER(KIND=JPIM), PARAMETER :: NGS13 = 134
INTEGER(KIND=JPIM), PARAMETER :: NGS14 = 136
INTEGER(KIND=JPIM), PARAMETER :: NGS15 = 138


INTEGER(KIND=JPIM) :: NGN(JPGMAX), NGBLW(JPGMAX)

!     ------------------------------------------------------------------
END MODULE YOERRTM

