INTERFACE
SUBROUTINE LWVB&
 & ( KIDIA , KFDIA , KLON , KLEV , KUAER,&
 & PABCU , PADJD , PADJU,&
 & PB , PBINT , PBSUR , PBTOP,&
 & PDISD , PDISU , PEMIS , PEMIW,&
 & PGASUR, PGBSUR, PGATOP, PGBTOP,&
 & PDWFSU,PFLUC&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOELW , ONLY : NSIL ,NIPD ,NTRA ,NUA ,NG1P1
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KUAER
REAL(KIND=JPRB) ,INTENT(IN) :: PABCU(KLON,NUA,3*KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PADJD(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PADJU(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PB(KLON,NSIL,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PBINT(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PBSUR(KLON,NSIL)
REAL(KIND=JPRB) ,INTENT(IN) :: PBTOP(KLON,NSIL)
REAL(KIND=JPRB) ,INTENT(IN) :: PDISD(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PDISU(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PEMIS(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PEMIW(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PGASUR(KLON,NIPD,2)
REAL(KIND=JPRB) ,INTENT(IN) :: PGBSUR(KLON,NIPD,2)
REAL(KIND=JPRB) ,INTENT(IN) :: PGATOP(KLON,NIPD,2)
REAL(KIND=JPRB) ,INTENT(IN) :: PGBTOP(KLON,NIPD,2)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PDWFSU(KLON,NSIL)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFLUC(KLON,2,KLEV+1)
END SUBROUTINE LWVB
END INTERFACE
