MODULE PARRINT

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!     PARAMETERS REQUIRED FOR RADIATION INTERPOLATION

INTEGER(KIND=JPIM), PARAMETER :: JPRADCW=2
INTEGER(KIND=JPIM), PARAMETER :: JPRADCE=2
INTEGER(KIND=JPIM), PARAMETER :: JPRADCT=JPRADCW+JPRADCE
INTEGER(KIND=JPIM), PARAMETER :: JPRADFW=1
INTEGER(KIND=JPIM), PARAMETER :: JPRADFE=1
END MODULE PARRINT
