!     ------------------------------------------------------------------
NAMELIST/NAMPHY1/&
&ALBGLA   , ALBMAX   , ALBMER   , ALBMIN   , ALCRIN   , ALRCN1   ,&
&ALRCN2   , EA       , EC2REF   , EMCRIN   , EMMGLA   , EMMMER   ,&
&EWFC     , EWWILT   , GA       , GC1      ,&
&GC1S1    , GC1S2    , GC1S3    , GC1S4    , GC1Y1    , GTSVAP   ,&
&GVEGMX   , GLAIMX   , GNEIMX   , GWPIMX   , GCGEL    ,&
&GC2      , GC2REF   , GC3      , GC31     , GC32     ,&
&GCZ0H    , GF1      , GF3      , GF4      , GWFC     , GWLEX    ,&
&GWLMX    , GWWILT   , G1B      ,&
&G1CGSAT  , G1C1SAT  , G1P      , G1WSAT   , G2B      , G2CGSAT  ,&
&G2C1SAT  , G2P      , G2WSAT   , G3CGSAT  , GSNC1    , GSNC2    ,&
&HSOL     , HSOLIWR  , HSOLIT0  , OMTPRO   , OMWPRO   , RC1MAX   ,&
&RCTGLA   , RCGMAX   , RCTVEG   , RD1      , RD2GLA   , RD2MER   ,&
&RGL      , RHOMAX   , RHOMIN   , RSMAX    , RTINER   , RZ0GLA   ,&
&RZ0MER   , RZHZ0G   , RZHZ0M   , TMERGL   , TOEXP    , TOLIN    ,&
&TREF4    , SODELX   , WCRIN    , WCRINC   , WCRING   , WNEW     ,&
&WPMX     , WSMX     , XCRINR   , XCRINV   ,&
&LIMC     , LIMW     , LC1VAP   ,&
&NTVGLA   , NTVMER   , GCGELS   , GVEGMXS  , GLAIMXS  , GNEIMXS  ,&
&ALB1     , ALB2     , RLAIMX   , RLAI     ,NCHSP
