INTERFACE
SUBROUTINE LWVD&
 & ( KIDIA, KFDIA, KLON , KLEV , KTRAER,&
 & PABCU, PDBDT,&
 & PGA , PGB,&
 & PCNTRB, PDISD, PDISU, PDWFSU&
 & ) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOELW , ONLY : NSIL ,NIPD ,NTRA ,NUA ,NG1P1
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KTRAER
REAL(KIND=JPRB) ,INTENT(IN) :: PABCU(KLON,NUA,3*KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PDBDT(KLON,NSIL,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PGA(KLON,NIPD,2,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PGB(KLON,NIPD,2,KLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PCNTRB(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PDISD(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PDISU(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PDWFSU(KLON,NSIL)
END SUBROUTINE LWVD
END INTERFACE
