! Replaces cvflag.h

MODULE cvflag_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: icvflag_Tpa, cvflag_grav, cvflag_ice, ok_optim_yield, ok_entrain, ok_homo_tend, &
            ok_convstop, ok_intermittent, cvflag_prec_eject, qsat_depends_on_qt, adiab_ascent_mass_flux_depends_on_ejectliq, keepbug_ice_frac

  LOGICAL cvflag_grav
  LOGICAL cvflag_ice
  LOGICAL ok_optim_yield
  LOGICAL ok_entrain
  LOGICAL ok_homo_tend
  LOGICAL ok_convstop
  LOGICAL ok_intermittent
  LOGICAL cvflag_prec_eject
  LOGICAL qsat_depends_on_qt
  LOGICAL adiab_ascent_mass_flux_depends_on_ejectliq
  LOGICAL keepbug_ice_frac
  INTEGER icvflag_Tpa

  !$OMP THREADPRIVATE(icvflag_Tpa, cvflag_grav, cvflag_ice, ok_optim_yield, ok_entrain, ok_homo_tend, &
  !$OMP      ok_convstop, ok_intermittent, cvflag_prec_eject, qsat_depends_on_qt, adiab_ascent_mass_flux_depends_on_ejectliq, keepbug_ice_frac)
END MODULE cvflag_mod_h
