!  A.I avril 2023

  subroutine ini_COSP(ref_liq_cosp0,ref_ice_cosp0,pctsrf_cosp0,zu10m_cosp0,zv10m_cosp0, &
                      zxtsol_cosp0,zx_rh_cosp0,cldfra_cosp0,rnebcon_cosp0,flwc_cosp0, &
                      fiwc_cosp0,prfl_cosp0,psfl_cosp0,pmflxr_cosp0,pmflxs_cosp0, &
                      mr_ozone_cosp0,cldtau_cosp0,cldemi_cosp0,JrNt_cosp0)

! Routine pour initialiser les champs input pour Cosp au  1er appel de celui-ci
!         Ce 1er appel sert uniquement a definir les axes verticaux pour les 
!         sortie Cosp

      use dimphy
      include "ini_COSP.h"

      ! Initialisations pour le 1er passage a Cosp
        ref_liq_cosp0=1.
        ref_ice_cosp0=1.
        pctsrf_cosp0=0.5
        zu10m_cosp0=1.
        zv10m_cosp0=1.
        zxtsol_cosp0=288.
        zx_rh_cosp0=1.
        cldfra_cosp0=1.
        rnebcon_cosp0=0.
        flwc_cosp0=0.
        fiwc_cosp0=0.
        prfl_cosp0(:,1:klev)=0.
        psfl_cosp0(:,1:klev)=0.
        pmflxr_cosp0(:,1:klev)=0.
        pmflxs_cosp0(:,1:klev)=0.
        mr_ozone_cosp0=0.
        cldtau_cosp0=0.
        cldemi_cosp0=0.
        JrNt_cosp0=0.

     end subroutine ini_COSP
