! Replaces fcg_gcssold.h

MODULE fcg_gcssold_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: imp_fcg_gcssold, ts_fcg_gcssold, Tp_fcg_gcssold, Tp_ini_gcssold, xTurb_fcg_gcssold

  LOGICAL :: imp_fcg_gcssold, ts_fcg_gcssold, Tp_fcg_gcssold
  LOGICAL :: Tp_ini_gcssold
  LOGICAL :: xTurb_fcg_gcssold

  !$OMP THREADPRIVATE(imp_fcg_gcssold, ts_fcg_gcssold, Tp_fcg_gcssold, Tp_ini_gcssold, xTurb_fcg_gcssold)
END MODULE fcg_gcssold_mod_h
