#ifdef CPP_GPUM
  MODULE s2s
    USE s2s_mod 
    INTEGER, PUBLIC, PARAMETER :: s2s_str_maxlen = 256
  END MODULE s2s
#else

! s2s wrapper
  MODULE s2s
  
    PRIVATE
    INTEGER, PUBLIC, PARAMETER :: s2s_str_maxlen = 256
    PUBLIC s2s_initialize, s2s_finalize, s2s_gpu_activated, s2s_register_activated

  CONTAINS

    SUBROUTINE s2s_initialize()
    END SUBROUTINE s2s_initialize
    
    SUBROUTINE s2s_finalize()
    END SUBROUTINE s2s_finalize

    FUNCTION s2s_gpu_activated()
      IMPLICIT NONE
      LOGICAL :: s2s_gpu_activated
      s2s_gpu_activated=.FALSE.
    END FUNCTION s2s_gpu_activated

    FUNCTION s2s_register_activated()
      IMPLICIT NONE
      LOGICAL :: s2s_register_activated
      s2s_register_activated=.FALSE.
    END FUNCTION s2s_register_activated

  END MODULE s2s

#endif  
