MODULE radepsi_mod_h
  IMPLICIT NONE; PRIVATE
  PUBLIC :: zeelog, zepsc, zepsco, zepscq, zepsct, zepscw, zepsec, zepscr, repsct

  REAL(KIND = 8) ZEELOG, ZEPSC, ZEPSCO, ZEPSCQ, ZEPSCT, ZEPSCW
  REAL(KIND = 8) ZEPSEC, ZEPSCR
  PARAMETER (ZEELOG = 1.E-07) !1.e-10 (not good for 32-bit machines)
  PARAMETER (ZEPSC = 1.E-20)
  PARAMETER (ZEPSCO = 1.E-10)
  PARAMETER (ZEPSCQ = 1.E-10)
  PARAMETER (ZEPSCT = 1.E-20)
  PARAMETER (ZEPSCW = 1.E-20)
  PARAMETER (ZEPSEC = 1.0E-12)
  PARAMETER (ZEPSCR = 1.0E-10)

  REAL(KIND = 8) REPSCT
  PARAMETER (REPSCT = 1.0E-10)
END MODULE radepsi_mod_h
