!     -----------------------------------------------------------------
NAMELIST/NAERAD/&
&  LERAD1H, LERADHS, LEPO3RA, LRADLB , LONEWSW &
&, LCCNL  , LCCNO  , LECSRAD, LRAYL  , LRRTM  , LSRTM  &
&, LHVOLCA, LNEWAER, LDIFFC , LNOTROAER &
&, LRADONDEM &
&, NICEOPT, NLIQOPT, NMCICA , NRADIP , NRADLP &
&, NAER   , NMODE  , NOZOCL , NINHOM , NLAYINH &
&, NOVLP  , NLW    , NSW    , NRADFR , NLNGR1H &
&, NRADPFR, NRADPLA, NRINT  , NRPROMA, NCSRADF &
&, NRADINT, NRADRES, CRTABLEDIR, CRTABLEFIL &
&, RCCNSEA, RCCNLND, NPERTAER, NPERTOZ &
&, RPERTOZ, RLWINHF, RSWINHF, RRe2De  &
&, RCCO2  , RCCH4  , RCN2O  , RCCFC11, RCCFC12  &
&, NHINCSOL,LECO2VAR,LHGHG  , NSCEN  &
&, LEDBUG  &
&, LUVPROC, LUVTDEP, LUVDBG , NUV    , NUVTIM , NRADUV , RUVLAM, RMUZUV
!     -----------------------------------------------------------------



