MODULE PARRRTM

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

!     ------------------------------------------------------------------
!     Parameters relevant to AER's RRTM-LW radiation scheme

!     980714  JJMorcrette
!     ------------------------------------------------------------------

INTEGER(KIND=JPIM), PARAMETER :: JPLAY  = 100

INTEGER(KIND=JPIM), PARAMETER :: JPG    = 16
INTEGER(KIND=JPIM), PARAMETER :: JPBAND = 16
INTEGER(KIND=JPIM), PARAMETER :: JPXSEC = 4
INTEGER(KIND=JPIM), PARAMETER :: JPINPX = 35
INTEGER(KIND=JPIM), PARAMETER :: JPGPT  = 140

INTEGER(KIND=JPIM), PARAMETER :: NG1  = 8
INTEGER(KIND=JPIM), PARAMETER :: NG2  = 14
INTEGER(KIND=JPIM), PARAMETER :: NG3  = 16
INTEGER(KIND=JPIM), PARAMETER :: NG4  = 14
INTEGER(KIND=JPIM), PARAMETER :: NG5  = 16
INTEGER(KIND=JPIM), PARAMETER :: NG6  = 8
INTEGER(KIND=JPIM), PARAMETER :: NG7  = 12
INTEGER(KIND=JPIM), PARAMETER :: NG8  = 8
INTEGER(KIND=JPIM), PARAMETER :: NG9  = 12
INTEGER(KIND=JPIM), PARAMETER :: NG10 = 6
INTEGER(KIND=JPIM), PARAMETER :: NG11 = 8
INTEGER(KIND=JPIM), PARAMETER :: NG12 = 8
INTEGER(KIND=JPIM), PARAMETER :: NG13 = 4
INTEGER(KIND=JPIM), PARAMETER :: NG14 = 2
INTEGER(KIND=JPIM), PARAMETER :: NG15 = 2
INTEGER(KIND=JPIM), PARAMETER :: NG16 = 8

INTEGER(KIND=JPIM), PARAMETER :: NGS1  = 8
INTEGER(KIND=JPIM), PARAMETER :: NGS2  = 22
INTEGER(KIND=JPIM), PARAMETER :: NGS3  = 38
INTEGER(KIND=JPIM), PARAMETER :: NGS4  = 52
INTEGER(KIND=JPIM), PARAMETER :: NGS5  = 68
INTEGER(KIND=JPIM), PARAMETER :: NGS6  = 76
INTEGER(KIND=JPIM), PARAMETER :: NGS7  = 88
INTEGER(KIND=JPIM), PARAMETER :: NGS8  = 96
INTEGER(KIND=JPIM), PARAMETER :: NGS9  = 108
INTEGER(KIND=JPIM), PARAMETER :: NGS10 = 114
INTEGER(KIND=JPIM), PARAMETER :: NGS11 = 122
INTEGER(KIND=JPIM), PARAMETER :: NGS12 = 130
INTEGER(KIND=JPIM), PARAMETER :: NGS13 = 134
INTEGER(KIND=JPIM), PARAMETER :: NGS14 = 136
INTEGER(KIND=JPIM), PARAMETER :: NGS15 = 138

INTEGER(KIND=JPIM), PARAMETER, DIMENSION(16) :: GP_PER_LWBAND = (/8,14,16,14,16,8,12,8,12,6,8,8,4,2,2,2/)   !FC

!     ------------------------------------------------------------------
END MODULE PARRRTM
