INTERFACE
SUBROUTINE RRTM_RRTM_140GP &
 & ( KIDIA , KFDIA , KLON , KLEV,&
 & PAER  , PAPH  , PAP,&
 & PTS   , PTH   , PT,&
 & P_ZEMIS , P_ZEMIW,&
 & PQ    , PCCO2 , POZN,&
 & PCLDF , PTAUCLD,&
 & PTAU_LW,&
 & PEMIT , PFLUX , PFLUC, PTCLEAR, &
 &  PTOAG, PTOACG )  !FC
USE PARKIND1 ,ONLY : JPIM ,JPRB
USE YOERAD   ,ONLY : NLW !--C.Kleinschmitt
USE PARRRTM , ONLY : JPBAND ,JPXSEC ,JPGPT ,JPLAY ,&
 & JPINPX  
!-NLW in clesphys now OB
! Temporary fix waiting for cleaner interface (or not)
USE clesphys_mod_h, ONLY: NSW
!!include "clesphys.h"
INTEGER(KIND=JPIM),INTENT(IN) :: KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN) :: KFDIA
REAL(KIND=JPRB) ,INTENT(IN) :: PAER(KLON,6,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PAPH(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PAP(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PTS(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PTH(KLON,KLEV+1)
REAL(KIND=JPRB) ,INTENT(IN) :: PT(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: P_ZEMIS(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: P_ZEMIW(KLON)
REAL(KIND=JPRB) ,INTENT(IN) :: PQ(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PCCO2
REAL(KIND=JPRB) ,INTENT(IN) :: POZN(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PCLDF(KLON,KLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PTAUCLD(KLON,KLEV,JPBAND)
!--C.Kleinschmitt
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAU_LW(KLON,KLEV,NLW) ! LW Optical depth of aerosols  
!--end
REAL(KIND=JPRB) ,INTENT(OUT) :: PEMIT(KLON)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFLUX(KLON,2,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PFLUC(KLON,2,KLEV+1)
REAL(KIND=JPRB) ,INTENT(OUT) :: PTOAG(KLON,JPGPT) !FC
REAL(KIND=JPRB) ,INTENT(OUT) :: PTOACG(KLON,JPGPT) !FC
REAL(KIND=JPRB) ,INTENT(OUT) :: PTCLEAR(KLON)
END SUBROUTINE RRTM_RRTM_140GP
END INTERFACE
