INTERFACE
SUBROUTINE GPXYB(KPROMA,KSTART,KPROF,KFLEV,PVDELB,PVC,&
 & PRES,PDELP,PRDELP,PLNPR,PALPH,PRTGR,&
 & PRPRES,PRPP) 
USE PARKIND1 ,ONLY : JPIM ,JPRB
INTEGER(KIND=JPIM),INTENT(IN) :: KPROMA
INTEGER(KIND=JPIM),INTENT(IN) :: KFLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KSTART
INTEGER(KIND=JPIM),INTENT(IN) :: KPROF
REAL(KIND=JPRB) ,INTENT(IN) :: PVDELB(KFLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PVC(KFLEV)
REAL(KIND=JPRB) ,INTENT(IN) :: PRES(KPROMA,0:KFLEV)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PDELP(KPROMA,KFLEV)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PRDELP(KPROMA,KFLEV)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PLNPR(KPROMA,KFLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PALPH(KPROMA,KFLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRTGR(KPROMA,KFLEV)
REAL(KIND=JPRB) ,INTENT(OUT) :: PRPRES(KPROMA,KFLEV)
REAL(KIND=JPRB) ,INTENT(INOUT) :: PRPP(KPROMA,KFLEV)
END SUBROUTINE GPXYB
END INTERFACE
